@@ -1,6 +1,6 @@
 # coding: utf-8
-
-from __future__ import unicode_literals
+# Copyright (c) Pymatgen Development Team.
+# Distributed under the terms of the MIT License.
 
 """
 This module provides various representations of transformed structures. A
@@ -8,34 +8,23 @@ TransformedStructure is a structure that has been modified by undergoing a
 series of transformations.
 """
 
-
-__author__ = "Shyue Ping Ong, Will Richards"
-__copyright__ = "Copyright 2012, The Materials Project"
-__version__ = "1.0"
-__maintainer__ = "Shyue Ping Ong"
-__email__ = "shyuep@gmail.com"
-__date__ = "Mar 2, 2012"
-
+import datetime
+import json
 import os
 import re
-import json
-import datetime
-from copy import deepcopy
+from warnings import warn
 
-from monty.json import MontyDecoder
+from monty.json import MontyDecoder, MSONable, jsanitize
 
 from pymatgen.core.structure import Structure
-from pymatgen.io.cifio import CifParser
-from pymatgen.io.vaspio.vasp_input import Poscar
-from pymatgen.serializers.json_coders import PMGSONable
-from pymatgen.matproj.snl import StructureNL
-
-from warnings import warn
+from pymatgen.io.cif import CifParser
+from pymatgen.io.vasp.inputs import Poscar
+from pymatgen.io.vasp.sets import MPRelaxSet
 
 dec = MontyDecoder()
 
 
-class TransformedStructure(PMGSONable):
+class TransformedStructure(MSONable):
     """
     Container object for new structures that include history of
     transformations.
@@ -44,8 +33,7 @@ class TransformedStructure(PMGSONable):
     associated transformation history.
     """
 
-    def __init__(self, structure, transformations=None, history=None,
-                 other_parameters=None):
+    def __init__(self, structure, transformations=None, history=None, other_parameters=None):
         """
         Initializes a transformed structure from a structure.
 
@@ -74,9 +62,8 @@ class TransformedStructure(PMGSONable):
         """
         if len(self.history) == 0:
             raise IndexError("Can't undo. Already at oldest change.")
-        if 'input_structure' not in self.history[-1]:
-            raise IndexError("Can't undo. Latest history has no "
-                             "input_structure")
+        if "input_structure" not in self.history[-1]:
+            raise IndexError("Can't undo. Latest history has no " "input_structure")
         h = self.history.pop()
         self._undone.append((h, self.final_structure))
         s = h["input_structure"]
@@ -98,14 +85,13 @@ class TransformedStructure(PMGSONable):
         self.final_structure = s
 
     def __getattr__(self, name):
-        s = object.__getattribute__(self, 'final_structure')
+        s = object.__getattribute__(self, "final_structure")
         return getattr(s, name)
 
     def __len__(self):
         return len(self.history)
 
-    def append_transformation(self, transformation, return_alternatives=False,
-                              clear_redo=True):
+    def append_transformation(self, transformation, return_alternatives=False, clear_redo=True):
         """
         Appends a transformation to the TransformedStructure.
 
@@ -126,7 +112,8 @@ class TransformedStructure(PMGSONable):
 
         if return_alternatives and transformation.is_one_to_many:
             ranked_list = transformation.apply_transformation(
-                self.final_structure, return_ranked_list=return_alternatives)
+                self.final_structure, return_ranked_list=return_alternatives
+            )
 
             input_structure = self.final_structure.as_dict()
             alts = []
@@ -138,8 +125,8 @@ class TransformedStructure(PMGSONable):
                 hdict["output_parameters"] = x
                 self.final_structure = s
                 d = self.as_dict()
-                d['history'].append(hdict)
-                d['final_structure'] = s.as_dict()
+                d["history"].append(hdict)
+                d["final_structure"] = s.as_dict()
                 alts.append(TransformedStructure.from_dict(d))
 
             x = ranked_list[0]
@@ -151,13 +138,14 @@ class TransformedStructure(PMGSONable):
             self.history.append(hdict)
             self.final_structure = s
             return alts
-        else:
-            s = transformation.apply_transformation(self.final_structure)
-            hdict = transformation.as_dict()
-            hdict["input_structure"] = self.final_structure.as_dict()
-            hdict["output_parameters"] = {}
-            self.history.append(hdict)
-            self.final_structure = s
+
+        s = transformation.apply_transformation(self.final_structure)
+        hdict = transformation.as_dict()
+        hdict["input_structure"] = self.final_structure.as_dict()
+        hdict["output_parameters"] = {}
+        self.history.append(hdict)
+        self.final_structure = s
+        return None
 
     def append_filter(self, structure_filter):
         """
@@ -172,8 +160,7 @@ class TransformedStructure(PMGSONable):
         hdict["input_structure"] = self.final_structure.as_dict()
         self.history.append(hdict)
 
-    def extend_transformations(self, transformations,
-                               return_alternatives=False):
+    def extend_transformations(self, transformations, return_alternatives=False):
         """
         Extends a sequence of transformations to the TransformedStructure.
 
@@ -185,51 +172,47 @@ class TransformedStructure(PMGSONable):
                 total number of structures to return.
         """
         for t in transformations:
-            self.append_transformation(t,
-                    return_alternatives=return_alternatives)
+            self.append_transformation(t, return_alternatives=return_alternatives)
 
-    def get_vasp_input(self, vasp_input_set, generate_potcar=True):
+    def get_vasp_input(self, vasp_input_set=MPRelaxSet, **kwargs):
         """
-        Returns VASP input as a dict of vaspio objects.
+        Returns VASP input as a dict of vasp objects.
 
         Args:
             vasp_input_set (pymatgen.io.vaspio_set.VaspInputSet): input set
                 to create vasp input files from structures
-            generate_potcar (bool): Set to False to generate a POTCAR.spec
-                file instead of a POTCAR, which contains the POTCAR labels
-                but not the actual POTCAR. Defaults to True.
         """
-        d = vasp_input_set.get_all_vasp_input(self.final_structure,
-                                              generate_potcar)
+        d = vasp_input_set(self.final_structure, **kwargs).get_vasp_input()
         d["transformations.json"] = json.dumps(self.as_dict())
         return d
 
-    def write_vasp_input(self, vasp_input_set, output_dir,
-                         create_directory=True):
-        """
+    def write_vasp_input(self, vasp_input_set=MPRelaxSet, output_dir=".", create_directory=True, **kwargs):
+        r"""
         Writes VASP input to an output_dir.
 
         Args:
             vasp_input_set:
                 pymatgen.io.vaspio_set.VaspInputSet like object that creates
                 vasp input files from structures
-            output_dir:
-                Directory to output files
-            create_directory:
-                Create the directory if not present. Defaults to True.
+            output_dir: Directory to output files
+            create_directory: Create the directory if not present. Defaults to
+                True.
+            **kwargs: All keyword args supported by the VASP input set.
         """
-        vasp_input_set.write_input(self.final_structure, output_dir,
-                                   make_dir_if_not_present=create_directory)
+        vasp_input_set(self.final_structure, **kwargs).write_input(output_dir, make_dir_if_not_present=create_directory)
         with open(os.path.join(output_dir, "transformations.json"), "w") as fp:
             json.dump(self.as_dict(), fp)
 
     def __str__(self):
-        output = ["Current structure", "------------",
-                  str(self.final_structure),
-                  "\nHistory",
-                  "------------"]
+        output = [
+            "Current structure",
+            "------------",
+            str(self.final_structure),
+            "\nHistory",
+            "------------",
+        ]
         for h in self.history:
-            h.pop('input_structure', None)
+            h.pop("input_structure", None)
             output.append(str(h))
         output.append("\nOther parameters")
         output.append("------------")
@@ -237,6 +220,12 @@ class TransformedStructure(PMGSONable):
         return "\n".join(output)
 
     def set_parameter(self, key, value):
+        """
+        Set a parameter
+
+        :param key: The string key
+        :param value: The value.
+        """
         self.other_parameters[key] = value
 
     @property
@@ -255,13 +244,11 @@ class TransformedStructure(PMGSONable):
         Copy of all structures in the TransformedStructure. A
         structure is stored after every single transformation.
         """
-        hstructs = [Structure.from_dict(s['input_structure'])
-                    for s in self.history if 'input_structure' in s]
+        hstructs = [Structure.from_dict(s["input_structure"]) for s in self.history if "input_structure" in s]
         return hstructs + [self.final_structure]
 
     @staticmethod
-    def from_cif_string(cif_string, transformations=None, primitive=True,
-                        occupancy_tolerance=1.):
+    def from_cif_string(cif_string, transformations=None, primitive=True, occupancy_tolerance=1.0):
         """
         Generates TransformedStructure from a cif string.
 
@@ -274,8 +261,8 @@ class TransformedStructure(PMGSONable):
             primitive (bool): Option to set if the primitive cell should be
                 extracted. Defaults to True. However, there are certain
                 instances where you might want to use a non-primitive cell,
-                e.g., if you are trying to generate all possible orderings of partial removals
-                or order a disordered structure.
+                e.g., if you are trying to generate all possible orderings of
+                partial removals or order a disordered structure.
             occupancy_tolerance (float): If total occupancy of a site is
                 between 1 and occupancy_tolerance, the occupancies will be
                 scaled down to 1.
@@ -284,7 +271,7 @@ class TransformedStructure(PMGSONable):
             TransformedStructure
         """
         parser = CifParser.from_string(cif_string, occupancy_tolerance)
-        raw_string = re.sub("'", "\"", cif_string)
+        raw_string = re.sub(r"'", '"', cif_string)
         cif_dict = parser.as_dict()
         cif_keys = list(cif_dict.keys())
         s = parser.get_structures(primitive)[0]
@@ -293,10 +280,12 @@ class TransformedStructure(PMGSONable):
             source = partial_cif["_database_code_ICSD"] + "-ICSD"
         else:
             source = "uploaded cif"
-        source_info = {"source": source,
-                       "datetime": str(datetime.datetime.now()),
-                       "original_file": raw_string,
-                       "cif_data": cif_dict[cif_keys[0]]}
+        source_info = {
+            "source": source,
+            "datetime": str(datetime.datetime.now()),
+            "original_file": raw_string,
+            "cif_data": cif_dict[cif_keys[0]],
+        }
         return TransformedStructure(s, transformations, history=[source_info])
 
     @staticmethod
@@ -311,13 +300,16 @@ class TransformedStructure(PMGSONable):
         """
         p = Poscar.from_string(poscar_string)
         if not p.true_names:
-            raise ValueError("Transformation can be craeted only from POSCAR "
-                             "strings with proper VASP5 element symbols.")
-        raw_string = re.sub("'", "\"", poscar_string)
+            raise ValueError(
+                "Transformation can be craeted only from POSCAR " "strings with proper VASP5 element symbols."
+            )
+        raw_string = re.sub(r"'", '"', poscar_string)
         s = p.structure
-        source_info = {"source": "POSCAR",
-                       "datetime": str(datetime.datetime.now()),
-                       "original_file": raw_string}
+        source_info = {
+            "source": "POSCAR",
+            "datetime": str(datetime.datetime.now()),
+            "original_file": raw_string,
+        }
         return TransformedStructure(s, transformations, history=[source_info])
 
     def as_dict(self):
@@ -327,10 +319,9 @@ class TransformedStructure(PMGSONable):
         d = self.final_structure.as_dict()
         d["@module"] = self.__class__.__module__
         d["@class"] = self.__class__.__name__
-        d["history"] = deepcopy(self.history)
-        d["version"] = __version__
+        d["history"] = jsanitize(self.history)
         d["last_modified"] = str(datetime.datetime.utcnow())
-        d["other_parameters"] = deepcopy(self.other_parameters)
+        d["other_parameters"] = jsanitize(self.other_parameters)
         return d
 
     @classmethod
@@ -339,23 +330,31 @@ class TransformedStructure(PMGSONable):
         Creates a TransformedStructure from a dict.
         """
         s = Structure.from_dict(d)
-        return cls(s, history=d["history"],
-                   other_parameters=d.get("other_parameters", None))
+        return cls(s, history=d["history"], other_parameters=d.get("other_parameters", None))
 
-    def to_snl(self, authors, projects=None, references='', remarks=None,
-               data=None, created_at=None):
+    def to_snl(self, authors, **kwargs):
+        """
+        Generate SNL from TransformedStructure.
+
+        :param authors: List of authors
+        :param **kwargs: All kwargs supported by StructureNL.
+        :return: StructureNL
+        """
         if self.other_parameters:
-            warn('Data in TransformedStructure.other_parameters discarded '
-                 'during type conversion to SNL')
+            warn("Data in TransformedStructure.other_parameters discarded " "during type conversion to SNL")
         hist = []
         for h in self.history:
-            snl_metadata = h.pop('_snl', {})
-            hist.append({'name' : snl_metadata.pop('name', 'pymatgen'),
-                         'url' : snl_metadata.pop('url',
-                                    'http://pypi.python.org/pypi/pymatgen'),
-                         'description' : h})
-        return StructureNL(self.final_structure, authors, projects, references,
-                           remarks, data, hist, created_at)
+            snl_metadata = h.pop("_snl", {})
+            hist.append(
+                {
+                    "name": snl_metadata.pop("name", "pymatgen"),
+                    "url": snl_metadata.pop("url", "http://pypi.python.org/pypi/pymatgen"),
+                    "description": h,
+                }
+            )
+        from pymatgen.util.provenance import StructureNL
+
+        return StructureNL(self.final_structure, authors, history=hist, **kwargs)
 
     @classmethod
     def from_snl(cls, snl):
@@ -371,6 +370,6 @@ class TransformedStructure(PMGSONable):
         hist = []
         for h in snl.history:
             d = h.description
-            d['_snl'] = {'url' : h.url, 'name' : h.name}
+            d["_snl"] = {"url": h.url, "name": h.name}
             hist.append(d)
         return cls(snl.structure, history=hist)
