@@ -1,6 +1,7 @@
 # coding: utf-8
+# Copyright (c) Pymatgen Development Team.
+# Distributed under the terms of the MIT License.
 
-from __future__ import unicode_literals
 
 """
 This module implements various transmuter classes.
@@ -12,8 +13,6 @@ It also includes the helper function, batch_write_vasp_input to generate an
 entire directory of vasp input files for running.
 """
 
-from six.moves import filter, map
-
 __author__ = "Shyue Ping Ong, Will Richards"
 __copyright__ = "Copyright 2012, The Materials Project"
 __version__ = "0.1"
@@ -23,13 +22,13 @@ __date__ = "Mar 4, 2012"
 
 import os
 import re
-import warnings
-
 from multiprocessing import Pool
+
 from pymatgen.alchemy.materials import TransformedStructure
+from pymatgen.io.vasp.sets import MPRelaxSet
 
 
-class StandardTransmuter(object):
+class StandardTransmuter:
     """
     An example of a Transmuter object, which performs a sequence of
     transformations on many structures to generate TransformedStructures.
@@ -39,8 +38,13 @@ class StandardTransmuter(object):
         List of all transformed structures.
     """
 
-    def __init__(self, transformed_structures, transformations=None,
-                 extend_collection=0, ncores=None):
+    def __init__(
+        self,
+        transformed_structures,
+        transformations=None,
+        extend_collection=0,
+        ncores=None,
+    ):
         """
         Initializes a transmuter from an initial list of
         :class:`pymatgen.alchemy.materials.TransformedStructure`.
@@ -63,21 +67,7 @@ class StandardTransmuter(object):
         self.ncores = ncores
         if transformations is not None:
             for trans in transformations:
-                self.append_transformation(trans,
-                                           extend_collection=extend_collection)
-
-    def get_transformed_structures(self):
-        """
-        Returns all TransformedStructures.
-
-        .. deprecated:: v2.1.0
-
-            Use transformed_structures attribute instead. Will be removed in
-            next version.
-        """
-        warnings.warn("Use transformed_structures attribute instead.",
-                      DeprecationWarning)
-        return self.transformed_structures
+                self.append_transformation(trans, extend_collection=extend_collection)
 
     def __getitem__(self, index):
         return self.transformed_structures[index]
@@ -108,8 +98,7 @@ class StandardTransmuter(object):
     def __len__(self):
         return len(self.transformed_structures)
 
-    def append_transformation(self, transformation, extend_collection=False,
-                              clear_redo=True):
+    def append_transformation(self, transformation, extend_collection=False, clear_redo=True):
         """
         Appends a transformation to all TransformedStructures.
 
@@ -131,21 +120,20 @@ class StandardTransmuter(object):
             structure
         """
         if self.ncores and transformation.use_multiprocessing:
-            p = Pool(self.ncores)
-            #need to condense arguments into single tuple to use map
-            z = map(
-                lambda x: (x, transformation, extend_collection, clear_redo),
-                self.transformed_structures)
-            new_tstructs = p.map(_apply_transformation, z, 1)
-            self.transformed_structures = []
-            for ts in new_tstructs:
-                self.transformed_structures.extend(ts)
+            with Pool(self.ncores) as p:
+                # need to condense arguments into single tuple to use map
+                z = map(
+                    lambda x: (x, transformation, extend_collection, clear_redo),
+                    self.transformed_structures,
+                )
+                new_tstructs = p.map(_apply_transformation, z, 1)
+                self.transformed_structures = []
+                for ts in new_tstructs:
+                    self.transformed_structures.extend(ts)
         else:
             new_structures = []
             for x in self.transformed_structures:
-                new = x.append_transformation(transformation,
-                                              extend_collection,
-                                              clear_redo=clear_redo)
+                new = x.append_transformation(transformation, extend_collection, clear_redo=clear_redo)
                 if new is not None:
                     new_structures.extend(new)
             self.transformed_structures.extend(new_structures)
@@ -172,33 +160,19 @@ class StandardTransmuter(object):
         def test_transformed_structure(ts):
             return structure_filter.test(ts.final_structure)
 
-        self.transformed_structures = list(filter(test_transformed_structure,
-                                            self.transformed_structures))
+        self.transformed_structures = list(filter(test_transformed_structure, self.transformed_structures))
         for ts in self.transformed_structures:
             ts.append_filter(structure_filter)
 
-    def write_vasp_input(self, vasp_input_set, output_dir,
-                         create_directory=True, subfolder=None,
-                         include_cif=False):
-        """
+    def write_vasp_input(self, **kwargs):
+        r"""
         Batch write vasp input for a sequence of transformed structures to
         output_dir, following the format output_dir/{formula}_{number}.
 
         Args:
-            vasp_input_set: pymatgen.io.vaspio_set.VaspInputSet to create
-                vasp input files from structures
-            output_dir: Directory to output files
-            create_directory (bool): Create the directory if not present.
-                Defaults to True.
-            subfolder: Callable to create subdirectory name from
-                transformed_structure. e.g.,
-                lambda x: x.other_parameters["tags"][0] to use the first tag.
-            include_cif (bool): Whether to output a CIF as well. CIF files
-                are generally better supported in visualization programs.
+            \\*\\*kwargs: All kwargs supported by batch_write_vasp_input.
         """
-        batch_write_vasp_input(self.transformed_structures, vasp_input_set,
-                               output_dir, create_directory, subfolder,
-                               include_cif)
+        batch_write_vasp_input(self.transformed_structures, **kwargs)
 
     def set_parameter(self, key, value):
         """
@@ -239,8 +213,7 @@ class StandardTransmuter(object):
                 transmuter.
         """
         if isinstance(tstructs_or_transmuter, self.__class__):
-            self.transformed_structures.extend(tstructs_or_transmuter
-                                               .transformed_structures)
+            self.transformed_structures.extend(tstructs_or_transmuter.transformed_structures)
         else:
             for ts in tstructs_or_transmuter:
                 assert isinstance(ts, TransformedStructure)
@@ -274,8 +247,7 @@ class CifTransmuter(StandardTransmuter):
     structures.
     """
 
-    def __init__(self, cif_string, transformations=None, primitive=True,
-                 extend_collection=False):
+    def __init__(self, cif_string, transformations=None, primitive=True, extend_collection=False):
         """
         Generates a Transmuter from a cif string, possibly
         containing multiple structures.
@@ -295,21 +267,18 @@ class CifTransmuter(StandardTransmuter):
         structure_data = []
         read_data = False
         for line in lines:
-            if re.match("^\s*data", line):
+            if re.match(r"^\s*data", line):
                 structure_data.append([])
                 read_data = True
             if read_data:
                 structure_data[-1].append(line)
         for data in structure_data:
-            tstruct = TransformedStructure.from_cif_string("\n".join(data), [],
-                                                           primitive)
+            tstruct = TransformedStructure.from_cif_string("\n".join(data), [], primitive)
             transformed_structures.append(tstruct)
-        StandardTransmuter.__init__(self, transformed_structures,
-                                    transformations, extend_collection)
+        super().__init__(transformed_structures, transformations, extend_collection)
 
     @staticmethod
-    def from_filenames(filenames, transformations=None, primitive=True,
-                       extend_collection=False):
+    def from_filenames(filenames, transformations=None, primitive=True, extend_collection=False):
         """
         Generates a TransformedStructureCollection from a cif, possibly
         containing multiple structures.
@@ -326,32 +295,33 @@ class CifTransmuter(StandardTransmuter):
         for fname in filenames:
             with open(fname, "r") as f:
                 allcifs.append(f.read())
-        return CifTransmuter("\n".join(allcifs), transformations,
-                             primitive=primitive,
-                             extend_collection=extend_collection)
+        return CifTransmuter(
+            "\n".join(allcifs),
+            transformations,
+            primitive=primitive,
+            extend_collection=extend_collection,
+        )
 
 
 class PoscarTransmuter(StandardTransmuter):
     """
     Generates a transmuter from a sequence of POSCARs.
-
-    Args:
-        poscar_string: List of POSCAR strings
-        transformations: New transformations to be applied to all
-            structures.
-        extend_collection: Whether to use more than one output structure
-            from one-to-many transformations.
     """
 
-    def __init__(self, poscar_string, transformations=None,
-                 extend_collection=False):
+    def __init__(self, poscar_string, transformations=None, extend_collection=False):
+        """
+        Args:
+            poscar_string: List of POSCAR strings
+            transformations: New transformations to be applied to all
+                structures.
+            extend_collection: Whether to use more than one output structure
+                from one-to-many transformations.
+        """
         tstruct = TransformedStructure.from_poscar_string(poscar_string, [])
-        StandardTransmuter.__init__(self, [tstruct], transformations,
-                                    extend_collection=extend_collection)
+        super().__init__([tstruct], transformations, extend_collection=extend_collection)
 
     @staticmethod
-    def from_filenames(poscar_filenames, transformations=None,
-                       extend_collection=False):
+    def from_filenames(poscar_filenames, transformations=None, extend_collection=False):
         """
         Convenient constructor to generates a POSCAR transmuter from a list of
         POSCAR filenames.
@@ -366,15 +336,19 @@ class PoscarTransmuter(StandardTransmuter):
         tstructs = []
         for filename in poscar_filenames:
             with open(filename, "r") as f:
-                tstructs.append(TransformedStructure
-                                .from_poscar_string(f.read(), []))
-        return StandardTransmuter(tstructs, transformations,
-                                  extend_collection=extend_collection)
+                tstructs.append(TransformedStructure.from_poscar_string(f.read(), []))
+        return StandardTransmuter(tstructs, transformations, extend_collection=extend_collection)
 
 
-def batch_write_vasp_input(transformed_structures, vasp_input_set, output_dir,
-                           create_directory=True, subfolder=None,
-                           include_cif=False):
+def batch_write_vasp_input(
+    transformed_structures,
+    vasp_input_set=MPRelaxSet,
+    output_dir=".",
+    create_directory=True,
+    subfolder=None,
+    include_cif=False,
+    **kwargs,
+):
     """
     Batch write vasp input for a sequence of transformed structures to
     output_dir, following the format output_dir/{group}/{formula}_{number}.
@@ -395,17 +369,15 @@ def batch_write_vasp_input(transformed_structures, vasp_input_set, output_dir,
             programs.
     """
     for i, s in enumerate(transformed_structures):
-        formula = re.sub("\s+", "", s.final_structure.formula)
+        formula = re.sub(r"\s+", "", s.final_structure.formula)
         if subfolder is not None:
             subdir = subfolder(s)
-            dirname = os.path.join(output_dir, subdir,
-                                   "{}_{}".format(formula, i))
+            dirname = os.path.join(output_dir, subdir, "{}_{}".format(formula, i))
         else:
             dirname = os.path.join(output_dir, "{}_{}".format(formula, i))
-        s.write_vasp_input(vasp_input_set, dirname,
-                           create_directory=create_directory)
+        s.write_vasp_input(vasp_input_set, dirname, create_directory=create_directory, **kwargs)
         if include_cif:
-            from pymatgen.io.cifio import CifWriter
+            from pymatgen.io.cif import CifWriter
 
             writer = CifWriter(s.final_structure)
             writer.write_file(os.path.join(dirname, "{}.cif".format(formula)))
@@ -426,8 +398,7 @@ def _apply_transformation(inputs):
         any new structures created by a one-to-many transformation)
     """
     ts, transformation, extend_collection, clear_redo = inputs
-    new = ts.append_transformation(transformation, extend_collection,
-                                   clear_redo=clear_redo)
+    new = ts.append_transformation(transformation, extend_collection, clear_redo=clear_redo)
     o = [ts]
     if new:
         o.extend(new)
