@@ -1,11 +1,24 @@
 # coding: utf-8
-
-from __future__ import division, unicode_literals
+# Copyright (c) Pymatgen Development Team.
+# Distributed under the terms of the MIT License.
 
 """
 This module provides classes for calculating the ewald sum of a structure.
 """
 
+import bisect
+from copy import copy, deepcopy
+from datetime import datetime
+from math import log, pi, sqrt
+from typing import Dict
+from warnings import warn
+
+import numpy as np
+from monty.json import MSONable
+from scipy.special import comb, erfc
+from scipy import constants
+
+from pymatgen.core.structure import Structure
 
 __author__ = "Shyue Ping Ong, William Davidson Richard"
 __copyright__ = "Copyright 2011, The Materials Project"
@@ -16,21 +29,18 @@ __email__ = "shyuep@gmail.com"
 __status__ = "Production"
 __date__ = "Aug 1 2012"
 
-from math import pi, sqrt, log, exp, erfc, factorial
-from datetime import datetime
-from copy import deepcopy, copy
-import bisect
 
-import numpy as np
-
-from pymatgen.core.physical_constants import ELECTRON_CHARGE, EPSILON_0
-
-
-class EwaldSummation(object):
+class EwaldSummation(MSONable):
     """
     Calculates the electrostatic energy of a periodic array of charges using
     the Ewald technique.
-    Ref: http://www.ee.duke.edu/~ayt/ewaldpaper/ewaldpaper.html
+
+
+    Ref:
+    Ewald summation techniques in perspective: a survey
+    Abdulnour Y. Toukmaji and John A. Board Jr.
+    DOI: 10.1016/0010-4655(96)00016-1
+    URL: http://www.ee.duke.edu/~ayt/ewaldpaper/ewaldpaper.html
 
     This matrix can be used to do fast calculations of ewald sums after species
     removal.
@@ -41,17 +51,25 @@ class EwaldSummation(object):
     """
 
     # Converts unit of q*q/r into eV
-    CONV_FACT = 1e10 * ELECTRON_CHARGE / (4 * pi * EPSILON_0)
+    CONV_FACT = 1e10 * constants.e / (4 * pi * constants.epsilon_0)
 
-    def __init__(self, structure, real_space_cut=None, recip_space_cut=None,
-                 eta=None, acc_factor=8.0):
+    def __init__(
+        self,
+        structure,
+        real_space_cut=None,
+        recip_space_cut=None,
+        eta=None,
+        acc_factor=12.0,
+        w=1 / sqrt(2),
+        compute_forces=False,
+    ):
         """
         Initializes and calculates the Ewald sum. Default convergence
         parameters have been specified, but you can override them if you wish.
 
         Args:
             structure (Structure): Input structure that must have proper
-                Specie on all sites, i.e. Element with oxidation state. Use
+                Species on all sites, i.e. Element with oxidation state. Use
                 Structure.add_oxidation_state... for example.
             real_space_cut (float): Real space cutoff radius dictating how
                 many terms are used in the real space sum. Defaults to None,
@@ -64,40 +82,51 @@ class EwaldSummation(object):
                 determine automatically.
             acc_factor (float): No. of significant figures each sum is
                 converged to.
+            w (float): Weight parameter, w, has been included that represents
+                the relative computational expense of calculating a term in
+                real and reciprocal space. Default of 0.7 reproduces result
+                similar to GULP 4.2. This has little effect on the total
+                energy, but may influence speed of computation in large
+                systems. Note that this parameter is used only when the
+                cutoffs are set to None.
+            compute_forces (bool): Whether to compute forces. False by
+                default since it is usually not needed.
         """
         self._s = structure
+        self._charged = abs(structure.charge) > 1e-8
         self._vol = structure.volume
+        self._compute_forces = compute_forces
 
         self._acc_factor = acc_factor
-
         # set screening length
-        self._eta = eta if eta \
-            else (len(structure) * 0.01 / self._vol) ** (1 / 3) * pi
+        self._eta = eta if eta else (len(structure) * w / (self._vol ** 2)) ** (1 / 3) * pi
         self._sqrt_eta = sqrt(self._eta)
 
         # acc factor used to automatically determine the optimal real and
         # reciprocal space cutoff radii
         self._accf = sqrt(log(10 ** acc_factor))
 
-        self._rmax = real_space_cut if real_space_cut \
-            else self._accf / self._sqrt_eta
-        self._gmax = recip_space_cut if recip_space_cut \
-            else 2 * self._sqrt_eta * self._accf
+        self._rmax = real_space_cut if real_space_cut else self._accf / self._sqrt_eta
+        self._gmax = recip_space_cut if recip_space_cut else 2 * self._sqrt_eta * self._accf
 
         # The next few lines pre-compute certain quantities and store them.
         # Ewald summation is rather expensive, and these shortcuts are
         # necessary to obtain several factors of improvement in speedup.
-        self._oxi_states = [compute_average_oxidation_state(site)
-                            for site in structure]
-        self._coords = np.array(self._s.cart_coords)
-        self._forces = np.zeros((len(structure), 3))
+        self._oxi_states = [compute_average_oxidation_state(site) for site in structure]
 
-        # Now we call the relevant private methods to calculate the reciprocal
-        # and real space terms.
-        (self._recip, recip_forces) = self._calc_recip()
-        (self._real, self._point, real_point_forces) = \
-            self._calc_real_and_point()
-        self._forces = recip_forces + real_point_forces
+        self._coords = np.array(self._s.cart_coords)
+
+        # Define the private attributes to lazy compute reciprocal and real
+        # space terms.
+        self._initialized = False
+        self._recip = None
+        self._real, self._point = None, None
+        self._forces = None
+
+        # Compute the correction for a charged cell
+        self._charged_cell_energy = (
+            -EwaldSummation.CONV_FACT / 2 * np.pi / structure.volume / self._eta * structure.charge ** 2
+        )
 
     def compute_partial_energy(self, removed_indices):
         """
@@ -127,10 +156,8 @@ class EwaldSummation(object):
 
         def find_match(site):
             for test_site in sub_structure:
-                frac_diff = abs(np.array(site.frac_coords)
-                                - np.array(test_site.frac_coords)) % 1
-                frac_diff = [abs(a) < tol or abs(a) > 1 - tol
-                             for a in frac_diff]
+                frac_diff = abs(np.array(site.frac_coords) - np.array(test_site.frac_coords)) % 1
+                frac_diff = [abs(a) < tol or abs(a) > 1 - tol for a in frac_diff]
                 if all(frac_diff):
                     return test_site
             return None
@@ -162,6 +189,9 @@ class EwaldSummation(object):
         """
         The reciprocal space energy.
         """
+        if not self._initialized:
+            self._calc_ewald_terms()
+            self._initialized = True
         return sum(sum(self._recip))
 
     @property
@@ -171,6 +201,9 @@ class EwaldSummation(object):
         corresponds to the interaction energy between site i and site j in
         reciprocal space.
         """
+        if not self._initialized:
+            self._calc_ewald_terms()
+            self._initialized = True
         return self._recip
 
     @property
@@ -178,6 +211,9 @@ class EwaldSummation(object):
         """
         The real space space energy.
         """
+        if not self._initialized:
+            self._calc_ewald_terms()
+            self._initialized = True
         return sum(sum(self._real))
 
     @property
@@ -186,6 +222,9 @@ class EwaldSummation(object):
         The real space energy matrix. Each matrix element (i, j) corresponds to
         the interaction energy between site i and site j in real space.
         """
+        if not self._initialized:
+            self._calc_ewald_terms()
+            self._initialized = True
         return self._real
 
     @property
@@ -193,6 +232,9 @@ class EwaldSummation(object):
         """
         The point energy.
         """
+        if not self._initialized:
+            self._calc_ewald_terms()
+            self._initialized = True
         return sum(self._point)
 
     @property
@@ -201,6 +243,9 @@ class EwaldSummation(object):
         The point space matrix. A diagonal matrix with the point terms for each
         site in the diagonal elements.
         """
+        if not self._initialized:
+            self._calc_ewald_terms()
+            self._initialized = True
         return self._point
 
     @property
@@ -208,17 +253,27 @@ class EwaldSummation(object):
         """
         The total energy.
         """
-        return sum(sum(self._recip)) + sum(sum(self._real)) + sum(self._point)
+        if not self._initialized:
+            self._calc_ewald_terms()
+            self._initialized = True
+        return sum(sum(self._recip)) + sum(sum(self._real)) + sum(self._point) + self._charged_cell_energy
 
     @property
     def total_energy_matrix(self):
         """
         The total energy matrix. Each matrix element (i, j) corresponds to the
         total interaction energy between site i and site j.
+
+        Note that this does not include the charged-cell energy, which is only important
+        when the simulation cell is not charge balanced.
         """
+        if not self._initialized:
+            self._calc_ewald_terms()
+            self._initialized = True
+
         totalenergy = self._recip + self._real
-        for i in range(len(self._point)):
-            totalenergy[i, i] += self._point[i]
+        for i, energy in enumerate(self._point):
+            totalenergy[i, i] += energy
         return totalenergy
 
     @property
@@ -227,8 +282,38 @@ class EwaldSummation(object):
         The forces on each site as a Nx3 matrix. Each row corresponds to a
         site.
         """
+        if not self._initialized:
+            self._calc_ewald_terms()
+            self._initialized = True
+
+        if not self._compute_forces:
+            raise AttributeError("Forces are available only if compute_forces is True!")
         return self._forces
 
+    def get_site_energy(self, site_index):
+        """Compute the energy for a single site in the structure
+
+        Args:
+            site_index (int): Index of site
+        ReturnS:
+            (float) - Energy of that site"""
+        if not self._initialized:
+            self._calc_ewald_terms()
+            self._initialized = True
+
+        if self._charged:
+            warn("Per atom energies for charged structures not supported in EwaldSummation")
+        return np.sum(self._recip[:, site_index]) + np.sum(self._real[:, site_index]) + self._point[site_index]
+
+    def _calc_ewald_terms(self):
+        """
+        Calculates and sets all ewald terms (point, real and reciprocal)
+        """
+        self._recip, recip_forces = self._calc_recip()
+        self._real, self._point, real_point_forces = self._calc_real_and_point()
+        if self._compute_forces:
+            self._forces = recip_forces + real_point_forces
+
     def _calc_recip(self):
         """
         Perform the reciprocal space summation. Calculates the quantity
@@ -242,95 +327,92 @@ class EwaldSummation(object):
         """
         numsites = self._s.num_sites
         prefactor = 2 * pi / self._vol
-        erecip = np.zeros((numsites, numsites))
-        forces = np.zeros((numsites, 3))
+        erecip = np.zeros((numsites, numsites), dtype=np.float_)
+        forces = np.zeros((numsites, 3), dtype=np.float_)
         coords = self._coords
         rcp_latt = self._s.lattice.reciprocal_lattice
-        recip_nn = rcp_latt.get_points_in_sphere([[0, 0, 0]], [0, 0, 0],
-                                                 self._gmax)
+        recip_nn = rcp_latt.get_points_in_sphere([[0, 0, 0]], [0, 0, 0], self._gmax)
 
-        frac_to_cart = rcp_latt.get_cartesian_coords
+        frac_coords = [fcoords for (fcoords, dist, i, img) in recip_nn if dist != 0]
+
+        gs = rcp_latt.get_cartesian_coords(frac_coords)
+        g2s = np.sum(gs ** 2, 1)
+        expvals = np.exp(-g2s / (4 * self._eta))
+        grs = np.sum(gs[:, None] * coords[None, :], 2)
 
         oxistates = np.array(self._oxi_states)
-        #create array where q_2[i,j] is qi * qj
+
+        # create array where q_2[i,j] is qi * qj
         qiqj = oxistates[None, :] * oxistates[:, None]
 
-        for (fcoords, dist, i) in recip_nn:
-            if dist == 0:
-                continue
-            gvect = frac_to_cart(fcoords)
-            gsquare = np.linalg.norm(gvect) ** 2
+        # calculate the structure factor
+        sreals = np.sum(oxistates[None, :] * np.cos(grs), 1)
+        simags = np.sum(oxistates[None, :] * np.sin(grs), 1)
 
-            expval = exp(-1.0 * gsquare / (4.0 * self._eta))
+        for g, g2, gr, expval, sreal, simag in zip(gs, g2s, grs, expvals, sreals, simags):
 
-            gvectdot = np.sum(gvect[None, :] * coords, 1)
+            # Uses the identity sin(x)+cos(x) = 2**0.5 sin(x + pi/4)
+            m = (gr[None, :] + pi / 4) - gr[:, None]
+            np.sin(m, m)
+            m *= expval / g2
 
-            #calculate the structure factor
-            sreal = np.sum(oxistates * np.cos(gvectdot))
-            simag = np.sum(oxistates * np.sin(gvectdot))
+            erecip += m
 
-            #create array where exparg[i,j] is gvectdot[i] - gvectdot[j]
-            exparg = gvectdot[None, :] - gvectdot[:, None]
+            if self._compute_forces:
+                pref = 2 * expval / g2 * oxistates
+                factor = prefactor * pref * (sreal * np.sin(gr) - simag * np.cos(gr))
 
-            #uses the identity sin(x)+cos(x) = 2**0.5 sin(x + pi/4)
-            sfactor = qiqj * np.sin(exparg + pi / 4) * 2 ** 0.5
+                forces += factor[:, None] * g[None, :]
 
-            erecip += expval / gsquare * sfactor
-            pref = 2 * expval / gsquare * oxistates
-            factor = prefactor * pref * \
-                (sreal * np.sin(gvectdot)
-                 - simag * np.cos(gvectdot)) * EwaldSummation.CONV_FACT
-
-            forces += factor[:, None] * gvect[None, :]
-
-        return erecip * prefactor * EwaldSummation.CONV_FACT, forces
+        forces *= EwaldSummation.CONV_FACT
+        erecip *= prefactor * EwaldSummation.CONV_FACT * qiqj * 2 ** 0.5
+        return erecip, forces
 
     def _calc_real_and_point(self):
         """
         Determines the self energy -(eta/pi)**(1/2) * sum_{i=1}^{N} q_i**2
-
-        If cell is charged a compensating background is added (i.e. a G=0 term)
         """
-        all_nn = self._s.get_all_neighbors(self._rmax, True)
-
+        fcoords = self._s.frac_coords
         forcepf = 2.0 * self._sqrt_eta / sqrt(pi)
         coords = self._coords
         numsites = self._s.num_sites
-        ereal = np.zeros((numsites, numsites))
-        epoint = np.zeros(numsites)
-        forces = np.zeros((numsites, 3))
+        ereal = np.empty((numsites, numsites), dtype=np.float_)
+
+        forces = np.zeros((numsites, 3), dtype=np.float_)
+
+        qs = np.array(self._oxi_states)
+
+        epoint = -(qs ** 2) * sqrt(self._eta / pi)
+
         for i in range(numsites):
-            nn = all_nn[i]  # self._s.get_neighbors(site, self._rmax)
-            num_neighbors = len(nn)
-            qi = self._oxi_states[i]
-            epoint[i] = qi * qi
-            epoint[i] *= -1.0 * sqrt(self._eta / pi)
-            # add jellium term
-            epoint[i] += qi * pi / (2.0 * self._vol * self._eta)
+            nfcoords, rij, js, _ = self._s.lattice.get_points_in_sphere(
+                fcoords, coords[i], self._rmax, zip_results=False
+            )
 
-            rij = np.zeros(num_neighbors)
-            qj = np.zeros(num_neighbors)
-            js = np.zeros(num_neighbors)
-            ncoords = np.zeros((num_neighbors, 3))
+            # remove the rii term
+            inds = rij > 1e-8
+            js = js[inds]
+            rij = rij[inds]
+            nfcoords = nfcoords[inds]
 
-            for k, (site, dist, j) in enumerate(nn):
-                rij[k] = dist
-                qj[k] = self._oxi_states[j]
-                js[k] = j
-                ncoords[k] = site.coords
+            qi = qs[i]
+            qj = qs[js]
 
-            erfcval = np.array([erfc(k) for k in self._sqrt_eta * rij])
+            erfcval = erfc(self._sqrt_eta * rij)
             new_ereals = erfcval * qi * qj / rij
 
-            #insert new_ereals
-            for k, new_e in enumerate(new_ereals):
-                ereal[js[k], i] += new_e
+            # insert new_ereals
+            for k in range(numsites):
+                ereal[k, i] = np.sum(new_ereals[js == k])
 
-            fijpf = qj / rij ** 3 * (erfcval + forcepf * rij *
-                                     np.exp(-self._eta * rij ** 2))
-            forces[i] += np.sum(np.expand_dims(fijpf, 1) *
-                                (np.array([coords[i]]) - ncoords) *
-                                qi * EwaldSummation.CONV_FACT, axis=0)
+            if self._compute_forces:
+                nccoords = self._s.lattice.get_cartesian_coords(nfcoords)
+
+                fijpf = qj / rij ** 3 * (erfcval + forcepf * rij * np.exp(-self._eta * rij ** 2))
+                forces[i] += np.sum(
+                    np.expand_dims(fijpf, 1) * (np.array([coords[i]]) - nccoords) * qi * EwaldSummation.CONV_FACT,
+                    axis=0,
+                )
 
         ereal *= 0.5 * EwaldSummation.CONV_FACT
         epoint *= EwaldSummation.CONV_FACT
@@ -338,16 +420,80 @@ class EwaldSummation(object):
 
     @property
     def eta(self):
+        """
+        Returns: eta value used in Ewald summation.
+        """
         return self._eta
 
     def __str__(self):
-        output = ["Real = " + str(self.real_space_energy),
-                  "Reciprocal = " + str(self.reciprocal_space_energy),
-                  "Point = " + str(self.point_energy),
-                  "Total = " + str(self.total_energy),
-                  "Forces:\n" + str(self.forces)]
+        if self._compute_forces:
+            output = [
+                "Real = " + str(self.real_space_energy),
+                "Reciprocal = " + str(self.reciprocal_space_energy),
+                "Point = " + str(self.point_energy),
+                "Total = " + str(self.total_energy),
+                "Forces:\n" + str(self.forces),
+            ]
+        else:
+            output = [
+                "Real = " + str(self.real_space_energy),
+                "Reciprocal = " + str(self.reciprocal_space_energy),
+                "Point = " + str(self.point_energy),
+                "Total = " + str(self.total_energy),
+                "Forces were not computed",
+            ]
         return "\n".join(output)
 
+    def as_dict(self, verbosity: int = 0) -> Dict:
+        """
+        Json-serialization dict representation of EwaldSummation.
+
+        Args:
+            verbosity (int): Verbosity level. Default of 0 only includes the
+                matrix representation. Set to 1 for more details.
+        """
+
+        d = {
+            "@module": self.__class__.__module__,
+            "@class": self.__class__.__name__,
+            "structure": self._s.as_dict(),
+            "compute_forces": self._compute_forces,
+            "eta": self._eta,
+            "acc_factor": self._acc_factor,
+            "real_space_cut": self._rmax,
+            "recip_space_cut": self._gmax,
+            "_recip": None if self._recip is None else self._recip.tolist(),
+            "_real": None if self._real is None else self._real.tolist(),
+            "_point": None if self._point is None else self._point.tolist(),
+            "_forces": None if self._forces is None else self._forces.tolist(),
+        }
+
+        return d
+
+    @classmethod
+    def from_dict(cls, d: Dict, fmt: str = None, **kwargs):
+        """
+        Create an EwaldSummation instance from json serialized dictionary.
+        """
+        summation = cls(
+            structure=Structure.from_dict(d["structure"]),
+            real_space_cut=d["real_space_cut"],
+            recip_space_cut=d["recip_space_cut"],
+            eta=d["eta"],
+            acc_factor=d["acc_factor"],
+            compute_forces=d["compute_forces"],
+        )
+
+        # set previously computed private attributes
+        if d["_recip"] is not None:
+            summation._recip = np.array(d["_recip"])
+            summation._real = np.array(d["_real"])
+            summation._point = np.array(d["_point"])
+            summation._forces = np.array(d["_forces"])
+            summation._initialized = True
+
+        return summation
+
 
 class EwaldMinimizer:
     """
@@ -365,21 +511,6 @@ class EwaldMinimizer:
     order disordered structure transformation.
 
     Author - Will Richards
-
-    Args:
-        matrix: A matrix of the ewald sum interaction energies. This is stored
-            in the class as a diagonally symmetric array and so
-            self._matrix will not be the same as the input matrix.
-        m_list: list of manipulations. each item is of the form
-            (multiplication fraction, number_of_indices, indices, species)
-            These are sorted such that the first manipulation contains the
-            most permutations. this is actually evaluated last in the
-            recursion since I'm using pop.
-        num_to_return: The minimizer will find the number_returned lowest
-            energy structures. This is likely to return a number of duplicate
-            structures so it may be necessary to overestimate and then
-            remove the duplicates later. (duplicate checking in this
-            process is extremely expensive)
     """
 
     ALGO_FAST = 0
@@ -394,6 +525,22 @@ class EwaldMinimizer:
     ALGO_TIME_LIMIT = 3
 
     def __init__(self, matrix, m_list, num_to_return=1, algo=ALGO_FAST):
+        """
+        Args:
+            matrix: A matrix of the ewald sum interaction energies. This is stored
+                in the class as a diagonally symmetric array and so
+                self._matrix will not be the same as the input matrix.
+            m_list: list of manipulations. each item is of the form
+                (multiplication fraction, number_of_indices, indices, species)
+                These are sorted such that the first manipulation contains the
+                most permutations. this is actually evaluated last in the
+                recursion since I'm using pop.
+            num_to_return: The minimizer will find the number_returned lowest
+                energy structures. This is likely to return a number of duplicate
+                structures so it may be necessary to overestimate and then
+                remove the duplicates later. (duplicate checking in this
+                process is extremely expensive)
+        """
         # Setup and checking of inputs
         self._matrix = copy(matrix)
         # Make the matrix diagonally symmetric (so matrix[i,:] == matrix[:,j])
@@ -403,22 +550,17 @@ class EwaldMinimizer:
                 self._matrix[i, j] = value
                 self._matrix[j, i] = value
 
-        def comb(n, k):
-            return factorial(n) / factorial(k) / factorial(n - k)
-
         # sort the m_list based on number of permutations
-        self._m_list = sorted(m_list, key=lambda x: comb(len(x[2]), x[1]),
-                              reverse=True)
+        self._m_list = sorted(m_list, key=lambda x: comb(len(x[2]), x[1]), reverse=True)
 
         for mlist in self._m_list:
             if mlist[0] > 1:
-                raise ValueError('multiplication fractions must be <= 1')
-        self._current_minimum = float('inf')
+                raise ValueError("multiplication fractions must be <= 1")
+        self._current_minimum = float("inf")
         self._num_to_return = num_to_return
         self._algo = algo
         if algo == EwaldMinimizer.ALGO_COMPLETE:
-            raise NotImplementedError('Complete algo not yet implemented for '
-                                      'EwaldMinimizer')
+            raise NotImplementedError("Complete algo not yet implemented for " "EwaldMinimizer")
 
         self._output_lists = []
         # Tag that the recurse function looks at at each level. If a method
@@ -437,10 +579,9 @@ class EwaldMinimizer:
         This method finds and returns the permutations that produce the lowest
         ewald sum calls recursive function to iterate through permutations
         """
-        if self._algo == EwaldMinimizer.ALGO_FAST or \
-                self._algo == EwaldMinimizer.ALGO_BEST_FIRST:
-            return self._recurse(self._matrix, self._m_list,
-                                 set(range(len(self._matrix))))
+        if self._algo == EwaldMinimizer.ALGO_FAST or self._algo == EwaldMinimizer.ALGO_BEST_FIRST:
+            return self._recurse(self._matrix, self._m_list, set(range(len(self._matrix))))
+        return None
 
     def add_m_list(self, matrix_sum, m_list):
         """
@@ -451,8 +592,7 @@ class EwaldMinimizer:
             self._output_lists = [[matrix_sum, m_list]]
         else:
             bisect.insort(self._output_lists, [matrix_sum, m_list])
-        if self._algo == EwaldMinimizer.ALGO_BEST_FIRST and \
-                len(self._output_lists) == self._num_to_return:
+        if self._algo == EwaldMinimizer.ALGO_BEST_FIRST and len(self._output_lists) == self._num_to_return:
             self._finished = True
         if len(self._output_lists) > self._num_to_return:
             self._output_lists.pop()
@@ -482,7 +622,7 @@ class EwaldMinimizer:
         interaction_matrix = matrix[indices, :][:, indices]
 
         fractions = np.zeros(len(interaction_matrix)) + 1
-        fractions[:len(fraction_list)] = fraction_list
+        fractions[: len(fraction_list)] = fraction_list
         fractions = np.sort(fractions)
 
         # Sum associated with each index (disregarding interactions between
@@ -506,19 +646,21 @@ class EwaldMinimizer:
             avg_frac = np.average(np.outer(1 - fractions, 1 - fractions))
             average_correction = avg_int * avg_frac
 
-            interaction_correction = average_correction * speedup_parameter \
-                + interaction_correction * (1 - speedup_parameter)
+            interaction_correction = average_correction * speedup_parameter + interaction_correction * (
+                1 - speedup_parameter
+            )
 
-        best_case = np.sum(matrix) + np.inner(sums[::-1], fractions - 1) \
-            + interaction_correction
+        best_case = np.sum(matrix) + np.inner(sums[::-1], fractions - 1) + interaction_correction
 
         return best_case
 
-    def get_next_index(self, matrix, manipulation, indices_left):
+    @classmethod
+    def get_next_index(cls, matrix, manipulation, indices_left):
         """
         Returns an index that should have the most negative effect on the
         matrix sum
         """
+        # pylint: disable=E1126
         f = manipulation[0]
         indices = list(indices_left.intersection(manipulation[2]))
         sums = np.sum(matrix[indices], axis=1)
@@ -541,22 +683,22 @@ class EwaldMinimizer:
             indices: Set of indices which haven't had a permutation
                 performed on them.
         """
-        #check to see if we've found all the solutions that we need
+        # check to see if we've found all the solutions that we need
         if self._finished:
             return
 
-        #if we're done with the current manipulation, pop it off.
+        # if we're done with the current manipulation, pop it off.
         while m_list[-1][1] == 0:
             m_list = copy(m_list)
             m_list.pop()
-            #if there are no more manipulations left to do check the value
+            # if there are no more manipulations left to do check the value
             if not m_list:
                 matrix_sum = np.sum(matrix)
                 if matrix_sum < self._current_minimum:
                     self.add_m_list(matrix_sum, output_m_list)
                 return
 
-        #if we wont have enough indices left, return
+        # if we wont have enough indices left, return
         if m_list[-1][1] > len(indices.intersection(m_list[-1][2])):
             return
 
@@ -581,21 +723,30 @@ class EwaldMinimizer:
         indices2.remove(index)
         m_list2[-1][1] -= 1
 
-        #recurse through both the modified and unmodified matrices
+        # recurse through both the modified and unmodified matrices
 
         self._recurse(matrix2, m_list2, indices2, output_m_list2)
         self._recurse(matrix, m_list, indices, output_m_list)
 
     @property
     def best_m_list(self):
+        """
+        Returns: Best m_list found.
+        """
         return self._best_m_list
 
     @property
     def minimized_sum(self):
+        """
+        Returns: Minimized sum
+        """
         return self._minimized_sum
 
     @property
     def output_lists(self):
+        """
+        Returns: output lists.
+        """
         return self._output_lists
 
 
@@ -610,15 +761,15 @@ def compute_average_oxidation_state(site):
         Average oxidation state of site.
     """
     try:
-        avg_oxi = sum([sp.oxi_state * occu
-                       for sp, occu in site.species_and_occu.items()
-                       if sp is not None])
+        avg_oxi = sum([sp.oxi_state * occu for sp, occu in site.species.items() if sp is not None])
         return avg_oxi
     except AttributeError:
         pass
     try:
         return site.charge
     except AttributeError:
-        raise ValueError("Ewald summation can only be performed on structures "
-                         "that are either oxidation state decorated or have "
-                         "site charges.")
+        raise ValueError(
+            "Ewald summation can only be performed on structures "
+            "that are either oxidation state decorated or have "
+            "site charges."
+        )
