@@ -1,6 +1,7 @@
 # coding: utf-8
+# Copyright (c) Pymatgen Development Team.
+# Distributed under the terms of the MIT License.
 
-from __future__ import division, unicode_literals
 
 """
 This module defines the abstract base classes for battery-related classes.
@@ -10,7 +11,6 @@ can be defined in a general way. The Abc for battery classes implements some of
 these common definitions to allow sharing of common logic between them.
 """
 
-
 __author__ = "Anubhav Jain, Shyue Ping Ong"
 __copyright__ = "Copyright 2012, The Materials Project"
 __version__ = "0.1"
@@ -19,57 +19,81 @@ __email__ = "shyuep@gmail.com"
 __date__ = "Feb 1, 2012"
 __status__ = "Beta"
 
-from collections import Sequence
-from abc import ABCMeta, abstractproperty
+from collections.abc import Sequence
+from dataclasses import dataclass
+from typing import Dict, Tuple
 
-from pymatgen.serializers.json_coders import PMGSONable
-from pymatgen.core.physical_constants import AVOGADROS_CONST
+from monty.json import MSONable
+from scipy.constants import N_A
+
+from pymatgen.core import Composition, Element
+from pymatgen.entries.computed_entries import ComputedEntry
 
 
-class AbstractVoltagePair(object):
+@dataclass
+class AbstractVoltagePair(MSONable):
     """
     An Abstract Base Class for a Voltage Pair.
+
+    Attributes:
+        voltage : Voltage of voltage pair.
+        mAh: Energy in mAh.
+        mass_charge: Mass of charged pair.
+        mass_discharge: Mass of discharged pair.
+        vol_charge: Vol of charged pair.
+        vol_discharge: Vol of discharged pair.
+        frac_charge: Frac of working ion in charged pair.
+        frac_discharge: Frac of working ion in discharged pair.
+        working_ion_entry: Working ion as an entry.
+        framework : The compositions of one formula unit of the host material
     """
-    __metaclass__ = ABCMeta
 
-    @abstractproperty
-    def voltage(self):
-        return self._voltage
+    voltage: float
+    mAh: float
+    mass_charge: float
+    mass_discharge: float
+    vol_charge: float
+    vol_discharge: float
+    frac_charge: float
+    frac_discharge: float
+    working_ion_entry: ComputedEntry
+    _framework_formula: str  # should be made into Composition whenever the as_dict and from dict are fixed
 
-    @abstractproperty
-    def mAh(self):
-        return self._mAh
+    def __post_init__(self):
+        # ensure the the frame work is a reduced composition
+        self._framework_formula = self.framework.reduced_formula
 
-    @abstractproperty
-    def mass_charge(self):
-        return self._mass_charge
+    @property
+    def working_ion(self) -> Element:
+        """
+        working ion as pymatgen Element object
+        """
+        return self.working_ion_entry.composition.elements[0]
 
-    @abstractproperty
-    def mass_discharge(self):
-        return self._mass_discharge
+    @property
+    def framework(self) -> Composition:
+        """
+        The composition object representing the framework
+        """
+        return Composition(self._framework_formula)
 
-    @abstractproperty
-    def vol_charge(self):
-        return self._vol_charge
+    @property
+    def x_charge(self) -> float:
+        """
+        The number of working ions per formula unit of host in the charged state
+        """
+        return self.frac_charge * self.framework.num_atoms / (1 - self.frac_charge)
 
-    @abstractproperty
-    def vol_discharge(self):
-        return self._vol_discharge
-
-    @abstractproperty
-    def frac_charge(self):
-        return self._frac_charge
-
-    @abstractproperty
-    def frac_discharge(self):
-        return self._frac_discharge
-
-    @abstractproperty
-    def working_ion_entry(self):
-        return self._working_ion_entry
+    @property
+    def x_discharge(self) -> float:
+        """
+        The number of working ions per formula unit of host in the discharged state
+        """
+        return self.frac_discharge * self.framework.num_atoms / (1 - self.frac_discharge)
 
 
-class AbstractElectrode(Sequence, PMGSONable):
+@dataclass
+class AbstractElectrode(Sequence, MSONable):
     """
     An Abstract Base Class representing an Electrode. It is essentially a
     sequence of VoltagePairs. Generally, subclasses only need to implement
@@ -106,30 +130,20 @@ class AbstractElectrode(Sequence, PMGSONable):
 
     Developers implementing a new battery (other than the two general ones
     already implemented) need to implement a VoltagePair and an Electrode.
+    Attributes:
+        voltage_pairs: Objects that represent each voltage step
+        working_ion: Representation of the working ion that only contains element type
+        working_ion_entry: Representation of the working_ion that contains the energy
+        framework: The compositions of one formula unit of the host material
     """
 
-    __metaclass__ = ABCMeta
+    voltage_pairs: Tuple[AbstractVoltagePair]
+    working_ion_entry: ComputedEntry
+    _framework_formula: str  # should be made into Composition whenever the as_dict and from dict are fixed
 
-    @abstractproperty
-    def voltage_pairs(self):
-        """
-        Returns all the VoltagePairs
-        """
-        return
-
-    @abstractproperty
-    def working_ion(self):
-        """
-        The working ion as an Element object
-        """
-        return
-
-    @abstractproperty
-    def working_ion_entry(self):
-        """
-        The working ion as an Entry object
-        """
-        return
+    def __post_init__(self):
+        # ensure the the frame work is a reduced composition
+        self._framework_formula = self.framework.reduced_formula
 
     def __getitem__(self, index):
         return self.voltage_pairs[index]
@@ -143,6 +157,34 @@ class AbstractElectrode(Sequence, PMGSONable):
     def __len__(self):
         return len(self.voltage_pairs)
 
+    @property
+    def working_ion(self):
+        """
+        working ion as pymatgen Element object
+        """
+        return self.working_ion_entry.composition.elements[0]
+
+    @property
+    def framework(self):
+        """
+        The composition object representing the framework
+        """
+        return Composition(self._framework_formula)
+
+    @property
+    def x_charge(self) -> float:
+        """
+        The number of working ions per formula unit of host in the charged state
+        """
+        return self.voltage_pairs[0].x_charge
+
+    @property
+    def x_discharge(self) -> float:
+        """
+        The number of working ions per formula unit of host in the discharged state
+        """
+        return self.voltage_pairs[-1].x_discharge
+
     @property
     def max_delta_volume(self):
         """
@@ -179,19 +221,46 @@ class AbstractElectrode(Sequence, PMGSONable):
         """
         Maximum absolute difference in adjacent voltage steps
         """
-        steps = [self.voltage_pairs[i].voltage
-                 - self.voltage_pairs[i + 1].voltage
-                 for i in range(len(self.voltage_pairs) - 1)]
+        steps = [
+            self.voltage_pairs[i].voltage - self.voltage_pairs[i + 1].voltage
+            for i in range(len(self.voltage_pairs) - 1)
+        ]
         return max(steps) if len(steps) > 0 else 0
 
     @property
     def normalization_mass(self):
+        """
+        Returns: Mass used for normalization. This is the mass of the discharged
+            electrode of the last voltage pair.
+        """
         return self.voltage_pairs[-1].mass_discharge
 
     @property
     def normalization_volume(self):
+        """
+        Returns: Mass used for normalization. This is the vol of the discharged
+            electrode of the last voltage pair.
+        """
         return self.voltage_pairs[-1].vol_discharge
 
+    def get_sub_electrodes(self, adjacent_only=True):
+        """
+        If this electrode contains multiple voltage steps, then it is possible
+        to use only a subset of the voltage steps to define other electrodes.
+        Must be implemented for each electrode object.
+        Args:
+            adjacent_only: Only return electrodes from compounds that are
+                adjacent on the convex hull, i.e. no electrodes returned
+                will have multiple voltage steps if this is set true
+
+        Returns:
+            A list of Electrode objects
+        """
+        NotImplementedError(
+            "The get_sub_electrodes function must be implemented for each concrete electrode "
+            f"class {self.__class__.__name__,}"
+        )
+
     def get_average_voltage(self, min_voltage=None, max_voltage=None):
         """
         Average voltage for path satisfying between a min and max voltage.
@@ -206,16 +275,14 @@ class AbstractElectrode(Sequence, PMGSONable):
             Average voltage in V across the insertion path (a subset of the
             path can be chosen by the optional arguments)
         """
-        pairs_in_range = self._select_in_voltage_range(min_voltage,
-                                                       max_voltage)
+        pairs_in_range = self._select_in_voltage_range(min_voltage, max_voltage)
         if len(pairs_in_range) == 0:
             return 0
         total_cap_in_range = sum([p.mAh for p in pairs_in_range])
         total_edens_in_range = sum([p.mAh * p.voltage for p in pairs_in_range])
         return total_edens_in_range / total_cap_in_range
 
-    def get_capacity_grav(self, min_voltage=None, max_voltage=None,
-                          use_overall_normalization=True):
+    def get_capacity_grav(self, min_voltage=None, max_voltage=None, use_overall_normalization=True):
         """
         Get the gravimetric capacity of the electrode.
 
@@ -233,15 +300,15 @@ class AbstractElectrode(Sequence, PMGSONable):
             Gravimetric capacity in mAh/g across the insertion path (a subset
             of the path can be chosen by the optional arguments).
         """
-        pairs_in_range = self._select_in_voltage_range(min_voltage,
-                                                       max_voltage)
-        normalization_mass = self.normalization_mass \
-            if use_overall_normalization or len(pairs_in_range) == 0 \
+        pairs_in_range = self._select_in_voltage_range(min_voltage, max_voltage)
+        normalization_mass = (
+            self.normalization_mass
+            if use_overall_normalization or len(pairs_in_range) == 0
             else pairs_in_range[-1].mass_discharge
+        )
         return sum([pair.mAh for pair in pairs_in_range]) / normalization_mass
 
-    def get_capacity_vol(self, min_voltage=None, max_voltage=None,
-                         use_overall_normalization=True):
+    def get_capacity_vol(self, min_voltage=None, max_voltage=None, use_overall_normalization=True):
         """
         Get the volumetric capacity of the electrode.
 
@@ -259,16 +326,15 @@ class AbstractElectrode(Sequence, PMGSONable):
             Volumetric capacity in mAh/cc across the insertion path (a subset
             of the path can be chosen by the optional arguments)
         """
-        pairs_in_range = self._select_in_voltage_range(min_voltage,
-                                                       max_voltage)
-        normalization_vol = self.normalization_volume \
-            if use_overall_normalization or len(pairs_in_range) == 0 \
+        pairs_in_range = self._select_in_voltage_range(min_voltage, max_voltage)
+        normalization_vol = (
+            self.normalization_volume
+            if use_overall_normalization or len(pairs_in_range) == 0
             else pairs_in_range[-1].vol_discharge
-        return sum([pair.mAh for pair in pairs_in_range]) / normalization_vol \
-            * 1e24 / AVOGADROS_CONST
+        )
+        return sum([pair.mAh for pair in pairs_in_range]) / normalization_vol * 1e24 / N_A
 
-    def get_specific_energy(self, min_voltage=None, max_voltage=None,
-                            use_overall_normalization=True):
+    def get_specific_energy(self, min_voltage=None, max_voltage=None, use_overall_normalization=True):
         """
         Returns the specific energy of the battery in mAh/g.
 
@@ -286,12 +352,11 @@ class AbstractElectrode(Sequence, PMGSONable):
             Specific energy in Wh/kg across the insertion path (a subset of
             the path can be chosen by the optional arguments)
         """
-        return self.get_capacity_grav(min_voltage, max_voltage,
-                                      use_overall_normalization) \
-            * self.get_average_voltage(min_voltage, max_voltage)
+        return self.get_capacity_grav(min_voltage, max_voltage, use_overall_normalization) * self.get_average_voltage(
+            min_voltage, max_voltage
+        )
 
-    def get_energy_density(self, min_voltage=None, max_voltage=None,
-                           use_overall_normalization=True):
+    def get_energy_density(self, min_voltage=None, max_voltage=None, use_overall_normalization=True):
         """
         Args:
             min_voltage (float): The minimum allowable voltage for a given
@@ -307,9 +372,9 @@ class AbstractElectrode(Sequence, PMGSONable):
             Energy density in Wh/L across the insertion path (a subset of the
             path can be chosen by the optional arguments).
         """
-        return self.get_capacity_vol(min_voltage, max_voltage,
-                                     use_overall_normalization) \
-            * self.get_average_voltage(min_voltage, max_voltage)
+        return self.get_capacity_vol(min_voltage, max_voltage, use_overall_normalization) * self.get_average_voltage(
+            min_voltage, max_voltage
+        )
 
     def _select_in_voltage_range(self, min_voltage=None, max_voltage=None):
         """
@@ -324,9 +389,44 @@ class AbstractElectrode(Sequence, PMGSONable):
         Returns:
             A list of VoltagePair objects
         """
-        min_voltage = min_voltage if min_voltage is not None \
-            else self.min_voltage
-        max_voltage = max_voltage if max_voltage is not None \
-            else self.max_voltage
-        return list(filter(lambda p: min_voltage <= p.voltage <= max_voltage,
-                           self.voltage_pairs))
+        min_voltage = min_voltage if min_voltage is not None else self.min_voltage
+        max_voltage = max_voltage if max_voltage is not None else self.max_voltage
+        return list(filter(lambda p: min_voltage <= p.voltage <= max_voltage, self.voltage_pairs))
+
+    def get_summary_dict(self, print_subelectrodes=True) -> Dict:
+        """
+        Generate a summary dict.
+
+        Args:
+            print_subelectrodes: Also print data on all the possible
+                subelectrodes.
+
+        Returns:
+            A summary of this electrode"s properties in dict format.
+        """
+
+        d = {
+            "average_voltage": self.get_average_voltage(),
+            "max_voltage": self.max_voltage,
+            "min_voltage": self.min_voltage,
+            "max_delta_volume": self.max_delta_volume,
+            "max_voltage_step": self.max_voltage_step,
+            "capacity_grav": self.get_capacity_grav(),
+            "capacity_vol": self.get_capacity_vol(),
+            "energy_grav": self.get_specific_energy(),
+            "energy_vol": self.get_energy_density(),
+            "working_ion": self.working_ion.symbol,
+            "nsteps": self.num_steps,
+            "fracA_charge": self.voltage_pairs[0].frac_charge,
+            "fracA_discharge": self.voltage_pairs[-1].frac_discharge,
+            "framework_formula": self._framework_formula,
+        }
+
+        if print_subelectrodes:
+
+            def f_dict(c):
+                return c.get_summary_dict(print_subelectrodes=False)
+
+            d["adj_pairs"] = list(map(f_dict, self.get_sub_electrodes(adjacent_only=True)))
+            d["all_pairs"] = list(map(f_dict, self.get_sub_electrodes(adjacent_only=False)))
+        return d
