@@ -1,28 +1,27 @@
 # coding: utf-8
-
-from __future__ import division, unicode_literals
+# Copyright (c) Pymatgen Development Team.
+# Distributed under the terms of the MIT License.
 
 """
 This module provides classes that operate on points or vectors in 3D space.
 """
 
-
-__author__ = "Shyue Ping Ong"
-__copyright__ = "Copyright 2011, The Materials Project"
-__version__ = "1.0"
-__maintainer__ = "Shyue Ping Ong"
-__email__ = "shyuep@gmail.com"
-__status__ = "Production"
-__date__ = "Sep 23, 2011"
+import re
+import string
+import warnings
+from math import cos, pi, sin, sqrt
 
 import numpy as np
-import re
-from math import sin, cos, pi, sqrt
+from monty.json import MSONable
 
-from pymatgen.serializers.json_coders import PMGSONable
+from pymatgen.electronic_structure.core import Magmom
+from pymatgen.util.string import transformation_to_string
+from pymatgen.util.typing import ArrayLike
+
+__author__ = "Shyue Ping Ong, Shyam Dwaraknath, Matthew Horton"
 
 
-class SymmOp(PMGSONable):
+class SymmOp(MSONable):
     """
     A symmetry operation in cartesian space. Consists of a rotation plus a
     translation. Implementation is as an affine transformation matrix of rank 4
@@ -33,7 +32,7 @@ class SymmOp(PMGSONable):
         A 4x4 numpy.array representing the symmetry operation.
     """
 
-    def __init__(self, affine_transformation_matrix, tol=0.01):
+    def __init__(self, affine_transformation_matrix: ArrayLike, tol=0.01):
         """
         Initializes the SymmOp from a 4x4 affine transformation matrix.
         In general, this constructor should not be used unless you are
@@ -52,11 +51,11 @@ class SymmOp(PMGSONable):
         self.tol = tol
 
     @staticmethod
-    def from_rotation_and_translation(rotation_matrix=((1, 0, 0),
-                                                       (0, 1, 0),
-                                                       (0, 0, 1)),
-                                      translation_vec=(0, 0, 0),
-                                      tol=0.1):
+    def from_rotation_and_translation(
+        rotation_matrix: ArrayLike = ((1, 0, 0), (0, 1, 0), (0, 0, 1)),
+        translation_vec: ArrayLike = (0, 0, 0),
+        tol=0.1,
+    ):
         """
         Creates a symmetry operation from a rotation matrix and a translation
         vector.
@@ -74,16 +73,14 @@ class SymmOp(PMGSONable):
         if rotation_matrix.shape != (3, 3):
             raise ValueError("Rotation Matrix must be a 3x3 numpy array.")
         if translation_vec.shape != (3,):
-            raise ValueError("Translation vector must be a rank 1 numpy array "
-                             "with 3 elements.")
+            raise ValueError("Translation vector must be a rank 1 numpy array " "with 3 elements.")
         affine_matrix = np.eye(4)
         affine_matrix[0:3][:, 0:3] = rotation_matrix
         affine_matrix[0:3][:, 3] = translation_vec
         return SymmOp(affine_matrix, tol)
 
     def __eq__(self, other):
-        return np.allclose(self.affine_matrix, other.affine_matrix,
-                           atol=self.tol)
+        return np.allclose(self.affine_matrix, other.affine_matrix, atol=self.tol)
 
     def __hash__(self):
         return 7
@@ -92,8 +89,12 @@ class SymmOp(PMGSONable):
         return self.__str__()
 
     def __str__(self):
-        output = ["Rot:", str(self.affine_matrix[0:3][:, 0:3]), "tau",
-                  str(self.affine_matrix[0:3][:, 3])]
+        output = [
+            "Rot:",
+            str(self.affine_matrix[0:3][:, 0:3]),
+            "tau",
+            str(self.affine_matrix[0:3][:, 3]),
+        ]
         return "\n".join(output)
 
     def operate(self, point):
@@ -123,7 +124,7 @@ class SymmOp(PMGSONable):
         affine_points = np.concatenate([points, np.ones(points.shape[:-1] + (1,))], axis=-1)
         return np.inner(affine_points, self.affine_matrix)[..., :-1]
 
-    def apply_rotation_only(self, vector):
+    def apply_rotation_only(self, vector: ArrayLike):
         """
         Vectors should only be operated by the rotation matrix and not the
         translation vector.
@@ -133,7 +134,30 @@ class SymmOp(PMGSONable):
         """
         return np.dot(self.rotation_matrix, vector)
 
-    def are_symmetrically_related(self, point_a, point_b, tol=0.001):
+    def transform_tensor(self, tensor: np.ndarray):
+        """
+        Applies rotation portion to a tensor. Note that tensor has to be in
+        full form, not the Voigt form.
+
+        Args:
+            tensor (numpy array): a rank n tensor
+
+        Returns:
+            Transformed tensor.
+        """
+        dim = tensor.shape
+        rank = len(dim)
+        assert all(i == 3 for i in dim)
+        # Build einstein sum string
+        lc = string.ascii_lowercase
+        indices = lc[:rank], lc[rank : 2 * rank]
+        einsum_string = ",".join([a + i for a, i in zip(*indices)])
+        einsum_string += ",{}->{}".format(*indices[::-1])
+        einsum_args = [self.rotation_matrix] * rank + [tensor]
+
+        return np.einsum(einsum_string, *einsum_args)
+
+    def are_symmetrically_related(self, point_a: ArrayLike, point_b: ArrayLike, tol: float = 0.001) -> bool:
         """
         Checks if two points are symmetrically related.
 
@@ -152,14 +176,14 @@ class SymmOp(PMGSONable):
         return False
 
     @property
-    def rotation_matrix(self):
+    def rotation_matrix(self) -> np.ndarray:
         """
         A 3x3 numpy.array representing the rotation matrix.
         """
         return self.affine_matrix[0:3][:, 0:3]
 
     @property
-    def translation_vector(self):
+    def translation_vector(self) -> np.ndarray:
         """
         A rank 1 numpy.array of dim 3 representing the translation vector.
         """
@@ -174,7 +198,7 @@ class SymmOp(PMGSONable):
         return SymmOp(new_matrix)
 
     @property
-    def inverse(self):
+    def inverse(self) -> "SymmOp":
         """
         Returns inverse of transformation.
         """
@@ -182,8 +206,9 @@ class SymmOp(PMGSONable):
         return SymmOp(invr)
 
     @staticmethod
-    def from_axis_angle_and_translation(axis, angle, angle_in_radians=False,
-                                        translation_vec=(0, 0, 0)):
+    def from_axis_angle_and_translation(
+        axis: ArrayLike, angle: float, angle_in_radians: bool = False, translation_vec: ArrayLike = (0, 0, 0)
+    ) -> "SymmOp":
         """
         Generates a SymmOp for a rotation about a given axis plus translation.
 
@@ -201,10 +226,7 @@ class SymmOp(PMGSONable):
         if isinstance(axis, (tuple, list)):
             axis = np.array(axis)
 
-        if isinstance(translation_vec, (tuple, list)):
-            vec = np.array(translation_vec)
-        else:
-            vec = translation_vec
+        vec = np.array(translation_vec)
 
         a = angle if angle_in_radians else angle * pi / 180
         cosa = cos(a)
@@ -224,7 +246,9 @@ class SymmOp(PMGSONable):
         return SymmOp.from_rotation_and_translation(r, vec)
 
     @staticmethod
-    def from_origin_axis_angle(origin, axis, angle, angle_in_radians=False):
+    def from_origin_axis_angle(
+        origin: ArrayLike, axis: ArrayLike, angle: float, angle_in_radians: bool = False
+    ) -> "SymmOp":
         """
         Generates a SymmOp for a rotation about a given axis through an
         origin.
@@ -241,48 +265,63 @@ class SymmOp(PMGSONable):
             SymmOp.
         """
         theta = angle * pi / 180 if not angle_in_radians else angle
-        a = origin[0]
-        b = origin[1]
-        c = origin[2]
-        u = axis[0]
-        v = axis[1]
-        w = axis[2]
+        a = origin[0]  # type: ignore
+        b = origin[1]  # type: ignore
+        c = origin[2]  # type: ignore
+        u = axis[0]  # type: ignore
+        v = axis[1]  # type: ignore
+        w = axis[2]  # type: ignore
         # Set some intermediate values.
-        u2 = u * u
-        v2 = v * v
-        w2 = w * w
+        u2 = u * u  # type: ignore
+        v2 = v * v  # type: ignore
+        w2 = w * w  # type: ignore
         cos_t = cos(theta)
         sin_t = sin(theta)
-        l2 = u2 + v2 + w2
-        l = sqrt(l2)
+        l2 = u2 + v2 + w2  # type: ignore
+        l = sqrt(l2)  # type: ignore
 
         # Build the matrix entries element by element.
-        m11 = (u2 + (v2 + w2) * cos_t) / l2
-        m12 = (u * v * (1 - cos_t) - w * l * sin_t) / l2
-        m13 = (u * w * (1 - cos_t) + v * l * sin_t) / l2
-        m14 = (a * (v2 + w2) - u * (b * v + c * w)
-               + (u * (b * v + c * w) - a * (v2 + w2)) * cos_t
-               + (b * w - c * v) * l * sin_t) / l2
+        m11 = (u2 + (v2 + w2) * cos_t) / l2  # type: ignore
+        m12 = (u * v * (1 - cos_t) - w * l * sin_t) / l2  # type: ignore
+        m13 = (u * w * (1 - cos_t) + v * l * sin_t) / l2  # type: ignore
+        m14 = (  # type: ignore
+            a * (v2 + w2)  # type: ignore
+            - u * (b * v + c * w)  # type: ignore
+            + (u * (b * v + c * w) - a * (v2 + w2)) * cos_t  # type: ignore
+            + (b * w - c * v) * l * sin_t  # type: ignore
+        ) / l2  # type: ignore
 
-        m21 = (u * v * (1 - cos_t) + w * l * sin_t) / l2
-        m22 = (v2 + (u2 + w2) * cos_t) / l2
-        m23 = (v * w * (1 - cos_t) - u * l * sin_t) / l2
-        m24 = (b * (u2 + w2) - v * (a * u + c * w)
-               + (v * (a * u + c * w) - b * (u2 + w2)) * cos_t
-               + (c * u - a * w) * l * sin_t) / l2
+        m21 = (u * v * (1 - cos_t) + w * l * sin_t) / l2  # type: ignore
+        m22 = (v2 + (u2 + w2) * cos_t) / l2  # type: ignore
+        m23 = (v * w * (1 - cos_t) - u * l * sin_t) / l2  # type: ignore
+        m24 = (  # type: ignore
+            b * (u2 + w2)  # type: ignore
+            - v * (a * u + c * w)  # type: ignore
+            + (v * (a * u + c * w) - b * (u2 + w2)) * cos_t  # type: ignore
+            + (c * u - a * w) * l * sin_t  # type: ignore
+        ) / l2  # type: ignore
 
-        m31 = (u * w * (1 - cos_t) - v * l * sin_t) / l2
-        m32 = (v * w * (1 - cos_t) + u * l * sin_t) / l2
-        m33 = (w2 + (u2 + v2) * cos_t) / l2
-        m34 = (c * (u2 + v2) - w * (a * u + b * v)
-               + (w * (a * u + b * v) - c * (u2 + v2)) * cos_t
-               + (a * v - b * u) * l * sin_t) / l2
+        m31 = (u * w * (1 - cos_t) - v * l * sin_t) / l2  # type: ignore
+        m32 = (v * w * (1 - cos_t) + u * l * sin_t) / l2  # type: ignore
+        m33 = (w2 + (u2 + v2) * cos_t) / l2  # type: ignore
+        m34 = (  # type: ignore
+            c * (u2 + v2)  # type: ignore
+            - w * (a * u + b * v)  # type: ignore
+            + (w * (a * u + b * v) - c * (u2 + v2)) * cos_t  # type: ignore
+            + (a * v - b * u) * l * sin_t  # type: ignore
+        ) / l2
 
-        return SymmOp([[m11, m12, m13, m14], [m21, m22, m23, m24],
-                       [m31, m32, m33, m34], [0, 0, 0, 1]])
+        return SymmOp(
+            [  # type: ignore
+                [m11, m12, m13, m14],
+                [m21, m22, m23, m24],
+                [m31, m32, m33, m34],
+                [0, 0, 0, 1],
+            ]
+        )
 
     @staticmethod
-    def reflection(normal, origin=(0, 0, 0)):
+    def reflection(normal: ArrayLike, origin: ArrayLike = (0, 0, 0)) -> "SymmOp":
         """
         Returns reflection symmetry operation.
 
@@ -295,7 +334,7 @@ class SymmOp(PMGSONable):
         Returns:
             SymmOp for the reflection about the plane
         """
-        #Normalize the normal vector first.
+        # Normalize the normal vector first.
         n = np.array(normal, dtype=float) / np.linalg.norm(normal)
 
         u, v, w = n
@@ -309,16 +348,14 @@ class SymmOp(PMGSONable):
         xy = -2 * u * v
         xz = -2 * u * w
         yz = -2 * v * w
-        mirror_mat = [[xx, xy, xz, 0], [xy, yy, yz, 0], [xz, yz, zz, 0],
-                      [0, 0, 0, 1]]
+        mirror_mat = [[xx, xy, xz, 0], [xy, yy, yz, 0], [xz, yz, zz, 0], [0, 0, 0, 1]]
 
         if np.linalg.norm(origin) > 1e-6:
-            mirror_mat = np.dot(np.linalg.inv(translation),
-                                np.dot(mirror_mat, translation))
+            mirror_mat = np.dot(np.linalg.inv(translation), np.dot(mirror_mat, translation))
         return SymmOp(mirror_mat)
 
     @staticmethod
-    def inversion(origin=(0, 0, 0)):
+    def inversion(origin: ArrayLike = (0, 0, 0)) -> "SymmOp":
         """
         Inversion symmetry operation about axis.
 
@@ -335,7 +372,7 @@ class SymmOp(PMGSONable):
         return SymmOp(mat)
 
     @staticmethod
-    def rotoreflection(axis, angle, origin=(0, 0, 0)):
+    def rotoreflection(axis: ArrayLike, angle: float, origin: ArrayLike = (0, 0, 0)) -> "SymmOp":
         """
         Returns a roto-reflection symmetry operation
 
@@ -353,52 +390,30 @@ class SymmOp(PMGSONable):
         m = np.dot(rot.affine_matrix, refl.affine_matrix)
         return SymmOp(m)
 
-    def as_dict(self):
-        d = {"@module": self.__class__.__module__,
-             "@class": self.__class__.__name__,
-             "matrix": self.affine_matrix.tolist(), "tolerance": self.tol}
-        return d
+    def as_dict(self) -> dict:
+        """
+        :return: MSONAble dict.
+        """
+        return {
+            "@module": self.__class__.__module__,
+            "@class": self.__class__.__name__,
+            "matrix": self.affine_matrix.tolist(),
+            "tolerance": self.tol,
+        }
 
-    def as_xyz_string(self):
+    def as_xyz_string(self) -> str:
         """
         Returns a string of the form 'x, y, z', '-x, -y, z',
         '-y+1/2, x+1/2, z+1/2', etc. Only works for integer rotation matrices
         """
-        xyz = ['x', 'y', 'z']
-        strings = []
-
         # test for invalid rotation matrix
-        if not np.all(np.isclose(self.rotation_matrix,
-                                 np.round(self.rotation_matrix))):
-            raise ValueError('Rotation matrix must be integer')
+        if not np.all(np.isclose(self.rotation_matrix, np.round(self.rotation_matrix))):
+            warnings.warn("Rotation matrix should be integer")
 
-        for r, t in zip(self.rotation_matrix, self.translation_vector):
-            symbols = []
-            for val, axis in zip(r, xyz):
-                val = int(round(val))
-                if val == 1:
-                    if symbols:
-                        symbols.append('+')
-                    symbols.append(axis)
-                elif val == -1:
-                    symbols.append('-' + axis)
-                elif val > 1:
-                    if symbols:
-                        symbols.append('+')
-                    symbols.append(str(val) + axis)
-                elif val < -1:
-                    symbols.append(str(val) + axis)
-            import fractions
-            f = fractions.Fraction(float(t)).limit_denominator()
-            if abs(f) > 1e-6:
-                if f > 0:
-                    symbols.append('+')
-                symbols.append(str(f))
-            strings.append("".join(symbols))
-        return ', '.join(strings)
+        return transformation_to_string(self.rotation_matrix, translation_vec=self.translation_vector, delim=", ")
 
     @staticmethod
-    def from_xyz_string(xyz_string):
+    def from_xyz_string(xyz_string: str) -> "SymmOp":
         """
         Args:
             xyz_string: string of the form 'x, y, z', '-x, -y, z',
@@ -409,25 +424,191 @@ class SymmOp(PMGSONable):
         rot_matrix = np.zeros((3, 3))
         trans = np.zeros(3)
         toks = xyz_string.strip().replace(" ", "").lower().split(",")
-        re_rot = re.compile("([+-]?)(\d*)/?(\d*)([x-z])")
-        re_trans = re.compile("([+-]?)(\d+)/?(\d*)(?![x-z])")
+        re_rot = re.compile(r"([+-]?)([\d\.]*)/?([\d\.]*)([x-z])")
+        re_trans = re.compile(r"([+-]?)([\d\.]+)/?([\d\.]*)(?![x-z])")
         for i, tok in enumerate(toks):
             # build the rotation matrix
             for m in re_rot.finditer(tok):
-                factor = -1 if m.group(1) == "-" else 1
+                factor = -1.0 if m.group(1) == "-" else 1.0
                 if m.group(2) != "":
-                    factor *= float(m.group(2)) / float(m.group(3)) \
-                        if m.group(3) != "" else float(m.group(2))
+                    factor *= float(m.group(2)) / float(m.group(3)) if m.group(3) != "" else float(m.group(2))
                 j = ord(m.group(4)) - 120
                 rot_matrix[i, j] = factor
             # build the translation vector
             for m in re_trans.finditer(tok):
                 factor = -1 if m.group(1) == "-" else 1
-                num = float(m.group(2)) / float(m.group(3)) \
-                    if m.group(3) != "" else float(m.group(2))
+                num = float(m.group(2)) / float(m.group(3)) if m.group(3) != "" else float(m.group(2))
                 trans[i] = num * factor
         return SymmOp.from_rotation_and_translation(rot_matrix, trans)
 
     @classmethod
-    def from_dict(cls, d):
+    def from_dict(cls, d) -> "SymmOp":
+        """
+        :param d: dict
+        :return: SymmOp from dict representation.
+        """
         return cls(d["matrix"], d["tolerance"])
+
+
+class MagSymmOp(SymmOp):
+    """
+    Thin wrapper around SymmOp to extend it to support magnetic symmetry
+    by including a  time reversal operator. Magnetic symmetry is similar
+    to conventional crystal symmetry, except symmetry is reduced by the
+    addition of a time reversal operator which acts on an atom's magnetic
+    moment.
+    """
+
+    def __init__(self, affine_transformation_matrix: ArrayLike, time_reversal: int, tol: float = 0.01):
+        """
+        Initializes the MagSymmOp from a 4x4 affine transformation matrix
+        and time reversal operator.
+        In general, this constructor should not be used unless you are
+        transferring rotations.  Use the static constructors instead to
+        generate a SymmOp from proper rotations and translation.
+
+        Args:
+            affine_transformation_matrix (4x4 array): Representing an
+                affine transformation.
+            time_reversal (int): 1 or -1
+            tol (float): Tolerance for determining if matrices are equal.
+        """
+        SymmOp.__init__(self, affine_transformation_matrix, tol=tol)
+        if time_reversal not in (-1, 1):
+            raise Exception(
+                "Time reversal operator not well defined: {0}, {1}".format(time_reversal, type(time_reversal))
+            )
+        self.time_reversal = time_reversal
+
+    def __eq__(self, other):
+        return np.allclose(self.affine_matrix, other.affine_matrix, atol=self.tol) and (
+            self.time_reversal == other.time_reversal
+        )
+
+    def __str__(self):
+        return self.as_xyzt_string()
+
+    def __repr__(self):
+        output = [
+            "Rot:",
+            str(self.affine_matrix[0:3][:, 0:3]),
+            "tau",
+            str(self.affine_matrix[0:3][:, 3]),
+            "Time reversal:",
+            str(self.time_reversal),
+        ]
+        return "\n".join(output)
+
+    def __hash__(self):
+        # useful for obtaining a set of unique MagSymmOps
+        hashable_value = tuple(self.affine_matrix.flatten()) + (self.time_reversal,)
+        return hashable_value.__hash__()
+
+    def operate_magmom(self, magmom):
+        """
+        Apply time reversal operator on the magnetic moment. Note that
+        magnetic moments transform as axial vectors, not polar vectors.
+
+        See 'Symmetry and magnetic structures', Rodríguez-Carvajal and
+        Bourée for a good discussion. DOI: 10.1051/epjconf/20122200010
+
+        Args:
+            magmom: Magnetic moment as electronic_structure.core.Magmom
+            class or as list or np array-like
+
+        Returns:
+            Magnetic moment after operator applied as Magmom class
+        """
+
+        magmom = Magmom(magmom)  # type casting to handle lists as input
+
+        transformed_moment = (
+            self.apply_rotation_only(magmom.global_moment) * np.linalg.det(self.rotation_matrix) * self.time_reversal
+        )
+
+        # retains input spin axis if different from default
+        return Magmom.from_global_moment_and_saxis(transformed_moment, magmom.saxis)
+
+    @classmethod
+    def from_symmop(cls, symmop, time_reversal) -> "MagSymmOp":
+        """
+        Initialize a MagSymmOp from a SymmOp and time reversal operator.
+
+        Args:
+            symmop (SymmOp): SymmOp
+            time_reversal (int): Time reversal operator, +1 or -1.
+
+        Returns:
+            MagSymmOp object
+        """
+        magsymmop = cls(symmop.affine_matrix, time_reversal, symmop.tol)
+        return magsymmop
+
+    @staticmethod
+    def from_rotation_and_translation_and_time_reversal(
+        rotation_matrix: ArrayLike = ((1, 0, 0), (0, 1, 0), (0, 0, 1)),
+        translation_vec: ArrayLike = (0, 0, 0),
+        time_reversal: int = 1,
+        tol: float = 0.1,
+    ) -> "MagSymmOp":
+        """
+        Creates a symmetry operation from a rotation matrix, translation
+        vector and time reversal operator.
+
+        Args:
+            rotation_matrix (3x3 array): Rotation matrix.
+            translation_vec (3x1 array): Translation vector.
+            time_reversal (int): Time reversal operator, +1 or -1.
+            tol (float): Tolerance to determine if rotation matrix is valid.
+
+        Returns:
+            MagSymmOp object
+        """
+        symmop = SymmOp.from_rotation_and_translation(
+            rotation_matrix=rotation_matrix, translation_vec=translation_vec, tol=tol
+        )
+        return MagSymmOp.from_symmop(symmop, time_reversal)
+
+    @staticmethod
+    def from_xyzt_string(xyzt_string: str) -> "MagSymmOp":
+        """
+        Args:
+            xyz_string: string of the form 'x, y, z, +1', '-x, -y, z, -1',
+                '-2y+1/2, 3x+1/2, z-y+1/2, +1', etc.
+        Returns:
+            MagSymmOp object
+        """
+        symmop = SymmOp.from_xyz_string(xyzt_string.rsplit(",", 1)[0])
+        try:
+            time_reversal = int(xyzt_string.rsplit(",", 1)[1])
+        except Exception:
+            raise Exception("Time reversal operator could not be parsed.")
+        return MagSymmOp.from_symmop(symmop, time_reversal)
+
+    def as_xyzt_string(self) -> str:
+        """
+        Returns a string of the form 'x, y, z, +1', '-x, -y, z, -1',
+        '-y+1/2, x+1/2, z+1/2, +1', etc. Only works for integer rotation matrices
+        """
+        xyzt_string = SymmOp.as_xyz_string(self)
+        return xyzt_string + ", {:+}".format(self.time_reversal)
+
+    def as_dict(self) -> dict:
+        """
+        :return: MSONABle dict
+        """
+        return {
+            "@module": self.__class__.__module__,
+            "@class": self.__class__.__name__,
+            "matrix": self.affine_matrix.tolist(),
+            "tolerance": self.tol,
+            "time_reversal": self.time_reversal,
+        }
+
+    @classmethod
+    def from_dict(cls, d: dict) -> "MagSymmOp":
+        """
+        :param d: dict
+        :return: MagneticSymmOp from dict representation.
+        """
+        return cls(d["matrix"], tol=d["tolerance"], time_reversal=d["time_reversal"])
