@@ -1,6 +1,6 @@
 # coding: utf-8
-
-from __future__ import division, unicode_literals
+# Copyright (c) Pymatgen Development Team.
+# Distributed under the terms of the MIT License.
 
 """
 This module implements a FloatWithUnit, which is a subclass of float. It
@@ -11,7 +11,12 @@ units are detected. An ArrayWithUnit is also implemented, which is a subclass
 of numpy's ndarray with similar unit features.
 """
 
-from six.moves import filter, zip
+import collections
+import numbers
+from functools import partial
+
+import numpy as np
+import scipy.constants as const
 
 __author__ = "Shyue Ping Ong, Matteo Giantomassi"
 __copyright__ = "Copyright 2011, The Materials Project"
@@ -20,26 +25,19 @@ __maintainer__ = "Shyue Ping Ong, Matteo Giantomassi"
 __status__ = "Production"
 __date__ = "Aug 30, 2013"
 
-import numpy as np
-import six
-
-import collections
-from numbers import Number
-import numbers
-from functools import partial
-from pymatgen.core.physical_constants import N_a, e
-import re
-
 """
 Some conversion factors
 """
-Ha_to_eV = 27.21138386
+Ha_to_eV = 1 / const.physical_constants["electron volt-hartree relationship"][0]
 eV_to_Ha = 1 / Ha_to_eV
 Ry_to_eV = Ha_to_eV / 2
-amu_to_kg = 1.660538921e-27
-mile_to_meters = 1609.347219
-bohr_to_angstrom = 0.5291772083
+amu_to_kg = const.physical_constants["atomic mass unit-kilogram relationship"][0]
+mile_to_meters = const.mile
+bohr_to_angstrom = const.physical_constants["Bohr radius"][0] * 1e10
 bohr_to_ang = bohr_to_angstrom
+ang_to_bohr = 1 / bohr_to_ang
+kCal_to_kJ = const.calorie
+kb = const.physical_constants["Boltzmann constant in eV/K"][0]
 
 """
 Definitions of supported units. Values below are essentially scaling and
@@ -65,82 +63,80 @@ BASE_UNITS = {
         "s": 1,
         "min": 60,
         "h": 3600,
+        "d": 3600 * 24,
     },
-    "current": {
-        "A": 1
-    },
+    "current": {"A": 1},
     "temperature": {
         "K": 1,
     },
-    "amount": {
-        "mol": 1,
-        "atom": 1 / N_a
-    },
-    "intensity": {
-        "cd": 1
-    },
+    "amount": {"mol": 1, "atom": 1 / const.N_A},
+    "intensity": {"cd": 1},
     "memory": {
         "byte": 1,
         "Kb": 1024,
-        "Mb": 1024**2,
-        "Gb": 1024**3,
-        "Tb": 1024**4,
+        "Mb": 1024 ** 2,
+        "Gb": 1024 ** 3,
+        "Tb": 1024 ** 4,
     },
 }
 
 # Accept kb, mb, gb ... as well.
 BASE_UNITS["memory"].update({k.lower(): v for k, v in BASE_UNITS["memory"].items()})
 
-
-#This current list are supported derived units defined in terms of powers of
-#SI base units and constants.
+# This current list are supported derived units defined in terms of powers of
+# SI base units and constants.
 DERIVED_UNITS = {
     "energy": {
-        "eV": {"kg": 1, "m": 2, "s": -2, e: 1},
-        "meV": {"kg": 1, "m": 2, "s": -2, e * 1e-3: 1},
-        "Ha": {"kg": 1, "m": 2, "s": -2, e * Ha_to_eV: 1},
-        "Ry": {"kg": 1, "m": 2, "s": -2, e * Ry_to_eV: 1},
+        "eV": {"kg": 1, "m": 2, "s": -2, const.e: 1},
+        "meV": {"kg": 1, "m": 2, "s": -2, const.e * 1e-3: 1},
+        "Ha": {"kg": 1, "m": 2, "s": -2, const.e * Ha_to_eV: 1},
+        "Ry": {"kg": 1, "m": 2, "s": -2, const.e * Ry_to_eV: 1},
         "J": {"kg": 1, "m": 2, "s": -2},
-        "kJ": {"kg": 1, "m": 2, "s": -2, 1000: 1}
+        "kJ": {"kg": 1, "m": 2, "s": -2, 1000: 1},
+        "kCal": {"kg": 1, "m": 2, "s": -2, 1000: 1, kCal_to_kJ: 1},
     },
     "charge": {
         "C": {"A": 1, "s": 1},
-        "e": {"A": 1, "s": 1, e: 1},
+        "e": {"A": 1, "s": 1, const.e: 1},
     },
     "force": {
-        "N": {"kg": 1, "m": 1, "s": -2}
+        "N": {"kg": 1, "m": 1, "s": -2},
+        "KN": {"kg": 1, "m": 1, "s": -2, 1000: 1},
+        "MN": {"kg": 1, "m": 1, "s": -2, 1e6: 1},
+        "GN": {"kg": 1, "m": 1, "s": -2, 1e9: 1},
+    },
+    "frequency": {
+        "Hz": {"s": -1},
+        "KHz": {"s": -1, 1000: 1},
+        "MHz": {"s": -1, 1e6: 1},
+        "GHz": {"s": -1, 1e9: 1},
+        "THz": {"s": -1, 1e12: 1},
     },
     "pressure": {
         "Pa": {"kg": 1, "m": -1, "s": -2},
         "KPa": {"kg": 1, "m": -1, "s": -2, 1000: 1},
         "MPa": {"kg": 1, "m": -1, "s": -2, 1e6: 1},
-        "GPa": {"kg": 1, "m": -1, "s": -2, 1e9: 1}
+        "GPa": {"kg": 1, "m": -1, "s": -2, 1e9: 1},
     },
     "power": {
-        "W": {"m": 2, "kg": 1, "s": -3}
+        "W": {"m": 2, "kg": 1, "s": -3},
+        "KW": {"m": 2, "kg": 1, "s": -3, 1000: 1},
+        "MW": {"m": 2, "kg": 1, "s": -3, 1e6: 1},
+        "GW": {"m": 2, "kg": 1, "s": -3, 1e9: 1},
     },
-    "emf": {
-        "V": {"m": 2, "kg": 1, "s": -3, "A": -1}
-    },
-    "capacitance": {
-        "F": {"m": -2, "kg": -1, "s": 4, "A": 2}
-    },
-    "resistance": {
-        "ohm": {"m": 2, "kg": 1, "s": -3, "A": -2}
-    },
-    "conductance": {
-        "S": {"m": -2, "kg": -1, "s": 3, "A": 2}
-    },
-    "magnetic_flux": {
-        "Wb": {"m": 2, "kg": 1, "s": -2, "A": -1}
-    }
+    "emf": {"V": {"m": 2, "kg": 1, "s": -3, "A": -1}},
+    "capacitance": {"F": {"m": -2, "kg": -1, "s": 4, "A": 2}},
+    "resistance": {"ohm": {"m": 2, "kg": 1, "s": -3, "A": -2}},
+    "conductance": {"S": {"m": -2, "kg": -1, "s": 3, "A": 2}},
+    "magnetic_flux": {"Wb": {"m": 2, "kg": 1, "s": -2, "A": -1}},
+    "cross_section": {"barn": {"m": 2, 1e-28: 1}, "mbarn": {"m": 2, 1e-31: 1}},
 }
 
-
-ALL_UNITS = dict(list(BASE_UNITS.items()) + list(DERIVED_UNITS.items()))
+ALL_UNITS = dict(list(BASE_UNITS.items()) + list(DERIVED_UNITS.items()))  # type: ignore
+SUPPORTED_UNIT_NAMES = tuple(i for d in ALL_UNITS.values() for i in d.keys())
 
 # Mapping unit name --> unit type (unit names must be unique).
-_UNAME2UTYPE = {}
+_UNAME2UTYPE = {}  # type: ignore
 for utype, d in ALL_UNITS.items():
     assert not set(d.keys()).intersection(_UNAME2UTYPE.keys())
     _UNAME2UTYPE.update({uname: utype for uname in d})
@@ -149,8 +145,7 @@ del utype, d
 
 def _get_si_unit(unit):
     unit_type = _UNAME2UTYPE[unit]
-    si_unit = filter(lambda k: BASE_UNITS[unit_type][k] == 1,
-                     BASE_UNITS[unit_type].keys())
+    si_unit = filter(lambda k: BASE_UNITS[unit_type][k] == 1, BASE_UNITS[unit_type].keys())
     return list(si_unit)[0], BASE_UNITS[unit_type][unit]
 
 
@@ -160,20 +155,22 @@ class UnitError(BaseException):
     """
 
 
-def check_mappings(u):
+def _check_mappings(u):
     for v in DERIVED_UNITS.values():
         for k2, v2 in v.items():
-            if all([v2.get(ku, 0) == vu for ku, vu in u.items()]) and \
-                    all([u.get(kv2, 0) == vv2 for kv2, vv2 in v2.items()]):
+            if all(v2.get(ku, 0) == vu for ku, vu in u.items()) and all(
+                u.get(kv2, 0) == vv2 for kv2, vv2 in v2.items()
+            ):
                 return {k2: 1}
     return u
 
 
-class Unit(collections.Mapping):
+class Unit(collections.abc.Mapping):
     """
     Represents a unit, e.g., "m" for meters, etc. Supports compound units.
     Only integer powers are supported for units.
     """
+
     Error = UnitError
 
     def __init__(self, unit_def):
@@ -188,16 +185,18 @@ class Unit(collections.Mapping):
                 space-separated.
         """
 
-        if isinstance(unit_def, six.string_types):
+        if isinstance(unit_def, str):
             unit = collections.defaultdict(int)
-            for m in re.finditer("([A-Za-z]+)\s*\^*\s*([\-0-9]*)", unit_def):
+            import re
+
+            for m in re.finditer(r"([A-Za-z]+)\s*\^*\s*([\-0-9]*)", unit_def):
                 p = m.group(2)
                 p = 1 if not p else int(p)
                 k = m.group(1)
                 unit[k] += p
         else:
             unit = {k: v for k, v in dict(unit_def).items() if v != 0}
-        self._unit = check_mappings(unit)
+        self._unit = _check_mappings(unit)
 
     def __mul__(self, other):
         new_units = collections.defaultdict(int)
@@ -234,11 +233,10 @@ class Unit(collections.Mapping):
         return len(self._unit)
 
     def __repr__(self):
-        sorted_keys = sorted(self._unit.keys(),
-                             key=lambda k: (-self._unit[k], k))
-        return " ".join(["{}^{}".format(k, self._unit[k])
-                         if self._unit[k] != 1 else k
-                         for k in sorted_keys if self._unit[k] != 0])
+        sorted_keys = sorted(self._unit.keys(), key=lambda k: (-self._unit[k], k))
+        return " ".join(
+            ["{}^{}".format(k, self._unit[k]) if self._unit[k] != 1 else k for k in sorted_keys if self._unit[k] != 0]
+        )
 
     def __str__(self):
         return self.__repr__()
@@ -259,7 +257,7 @@ class Unit(collections.Mapping):
             for d in DERIVED_UNITS.values():
                 if k in d:
                     for k2, v2 in d[k].items():
-                        if isinstance(k2, Number):
+                        if isinstance(k2, numbers.Number):
                             factor *= k2 ** (v2 * v)
                         else:
                             b[k2] += v2 * v
@@ -282,10 +280,8 @@ class Unit(collections.Mapping):
         """
         uo_base, ofactor = self.as_base_units
         un_base, nfactor = Unit(new_unit).as_base_units
-        units_new = sorted(un_base.items(),
-                           key=lambda d: _UNAME2UTYPE[d[0]])
-        units_old = sorted(uo_base.items(),
-                           key=lambda d: _UNAME2UTYPE[d[0]])
+        units_new = sorted(un_base.items(), key=lambda d: _UNAME2UTYPE[d[0]])
+        units_old = sorted(uo_base.items(), key=lambda d: _UNAME2UTYPE[d[0]])
         factor = ofactor / nfactor
         for uo, un in zip(units_old, units_new):
             if uo[1] != un[1]:
@@ -317,6 +313,7 @@ class FloatWithUnit(float):
     >>> c.to("eV")
     32.932522246000005 eV
     """
+
     Error = UnitError
 
     @classmethod
@@ -324,7 +321,7 @@ class FloatWithUnit(float):
         """
         Initialize a FloatWithUnit from a string. Example Memory.from_string("1. Mb")
         """
-        # Extract num and unit string. 
+        # Extract num and unit string.
         s = s.strip()
         for i, char in enumerate(s):
             if char.isalpha() or char.isspace():
@@ -343,6 +340,7 @@ class FloatWithUnit(float):
         return cls(num, unit, unit_type=unit_type)
 
     def __new__(cls, val, unit, unit_type=None):
+        """Overrides __new__ since we are subclassing a Python primitive/"""
         new = float.__new__(cls, val)
         new._unit = Unit(unit)
         new._unit_type = unit_type
@@ -358,84 +356,62 @@ class FloatWithUnit(float):
             unit_type (str): A type of unit. E.g., "charge"
         """
         if unit_type is not None and str(unit) not in ALL_UNITS[unit_type]:
-            raise UnitError(
-                "{} is not a supported unit for {}".format(unit, unit_type))
+            raise UnitError("{} is not a supported unit for {}".format(unit, unit_type))
         self._unit = Unit(unit)
         self._unit_type = unit_type
 
     def __repr__(self):
-        return super(FloatWithUnit, self).__repr__()
+        return super().__repr__()
 
     def __str__(self):
-        s = super(FloatWithUnit, self).__str__()
+        s = super().__str__()
         return "{} {}".format(s, self._unit)
 
     def __add__(self, other):
         if not hasattr(other, "unit_type"):
-            return super(FloatWithUnit, self).__add__(other)
+            return super().__add__(other)
         if other.unit_type != self._unit_type:
             raise UnitError("Adding different types of units is not allowed")
         val = other
         if other.unit != self._unit:
             val = other.to(self._unit)
-        return FloatWithUnit(float(self) + val, unit_type=self._unit_type,
-                             unit=self._unit)
+        return FloatWithUnit(float(self) + val, unit_type=self._unit_type, unit=self._unit)
 
     def __sub__(self, other):
         if not hasattr(other, "unit_type"):
-            return super(FloatWithUnit, self).__sub__(other)
+            return super().__sub__(other)
         if other.unit_type != self._unit_type:
             raise UnitError("Subtracting different units is not allowed")
         val = other
         if other.unit != self._unit:
             val = other.to(self._unit)
-        return FloatWithUnit(float(self) - val, unit_type=self._unit_type,
-                             unit=self._unit)
+        return FloatWithUnit(float(self) - val, unit_type=self._unit_type, unit=self._unit)
 
     def __mul__(self, other):
         if not isinstance(other, FloatWithUnit):
-            return FloatWithUnit(float(self) * other,
-                                 unit_type=self._unit_type,
-                                 unit=self._unit)
-        return FloatWithUnit(float(self) * other, unit_type=None,
-                             unit=self._unit * other._unit)
+            return FloatWithUnit(float(self) * other, unit_type=self._unit_type, unit=self._unit)
+        return FloatWithUnit(float(self) * other, unit_type=None, unit=self._unit * other._unit)
 
     def __rmul__(self, other):
         if not isinstance(other, FloatWithUnit):
-            return FloatWithUnit(float(self) * other,
-                                 unit_type=self._unit_type,
-                                 unit=self._unit)
-        return FloatWithUnit(float(self) * other, unit_type=None,
-                             unit=self._unit * other._unit)
+            return FloatWithUnit(float(self) * other, unit_type=self._unit_type, unit=self._unit)
+        return FloatWithUnit(float(self) * other, unit_type=None, unit=self._unit * other._unit)
 
     def __pow__(self, i):
-        return FloatWithUnit(float(self) ** i, unit_type=None,
-                             unit=self._unit ** i)
-
-    def __div__(self, other):
-        val = super(FloatWithUnit, self).__div__(other)
-        if not isinstance(other, FloatWithUnit):
-            return FloatWithUnit(val, unit_type=self._unit_type,
-                                 unit=self._unit)
-        return FloatWithUnit(val, unit_type=None,
-                             unit=self._unit / other._unit)
+        return FloatWithUnit(float(self) ** i, unit_type=None, unit=self._unit ** i)
 
     def __truediv__(self, other):
-        val = super(FloatWithUnit, self).__truediv__(other)
+        val = super().__truediv__(other)
         if not isinstance(other, FloatWithUnit):
-            return FloatWithUnit(val, unit_type=self._unit_type,
-                                 unit=self._unit)
-        return FloatWithUnit(val, unit_type=None,
-                             unit=self._unit / other._unit)
+            return FloatWithUnit(val, unit_type=self._unit_type, unit=self._unit)
+        return FloatWithUnit(val, unit_type=None, unit=self._unit / other._unit)
 
     def __neg__(self):
-        return FloatWithUnit(super(FloatWithUnit, self).__neg__(),
-                             unit_type=self._unit_type,
-                             unit=self._unit)
+        return FloatWithUnit(super().__neg__(), unit_type=self._unit_type, unit=self._unit)
 
     def __getnewargs__(self):
         """Function used by pickle to recreate object."""
-        #print(self.__dict__)
+        # print(self.__dict__)
         # FIXME
         # There's a problem with _unit_type if we try to unpickle objects from file.
         # since self._unit_type might not be defined. I think this is due to
@@ -451,19 +427,25 @@ class FloatWithUnit(float):
     def __getstate__(self):
         state = self.__dict__.copy()
         state["val"] = float(self)
-        #print("in getstate %s" % state)
+        # print("in getstate %s" % state)
         return state
 
     def __setstate__(self, state):
-        #print("in setstate %s" % state)
+        # print("in setstate %s" % state)
         self._unit = state["_unit"]
 
     @property
-    def unit_type(self):
+    def unit_type(self) -> str:
+        """
+        :return: The type of unit. Energy, Charge, etc.
+        """
         return self._unit_type
 
     @property
-    def unit(self):
+    def unit(self) -> str:
+        """
+        :return: The unit, e.g., "eV".
+        """
         return self._unit
 
     def to(self, new_unit):
@@ -486,7 +468,18 @@ class FloatWithUnit(float):
         return FloatWithUnit(
             self * self.unit.get_conversion_factor(new_unit),
             unit_type=self._unit_type,
-            unit=new_unit)
+            unit=new_unit,
+        )
+
+    @property
+    def as_base_units(self):
+        """
+        Returns this FloatWithUnit in base SI units, including derived units.
+
+        Returns:
+            A FloatWithUnit object in base SI units
+        """
+        return self.to(self.unit.as_base_units[0])
 
     @property
     def supported_units(self):
@@ -514,9 +507,13 @@ class ArrayWithUnit(np.ndarray):
     >>> c.to("eV")
     array([ 28.21138386,  56.42276772]) eV
     """
+
     Error = UnitError
 
     def __new__(cls, input_array, unit, unit_type=None):
+        """
+        Override __new__.
+        """
         # Input array is an already formed ndarray instance
         # We first cast to be our class type
         obj = np.asarray(input_array).view(cls)
@@ -535,27 +532,31 @@ class ArrayWithUnit(np.ndarray):
         self._unit = getattr(obj, "_unit", None)
         self._unit_type = getattr(obj, "_unit_type", None)
 
-    #TODO abstract base class property?
     @property
-    def unit_type(self):
+    def unit_type(self) -> str:
+        """
+        :return: The type of unit. Energy, Charge, etc.
+        """
         return self._unit_type
 
-    #TODO abstract base class property?
     @property
-    def unit(self):
+    def unit(self) -> str:
+        """
+        :return: The unit, e.g., "eV".
+        """
         return self._unit
 
     def __reduce__(self):
-        #print("in reduce")
-        reduce = list(super(ArrayWithUnit, self).__reduce__())
-        #print("unit",self._unit)
-        #print(reduce[2])
+        # print("in reduce")
+        reduce = list(super().__reduce__())
+        # print("unit",self._unit)
+        # print(reduce[2])
         reduce[2] = {"np_state": reduce[2], "_unit": self._unit}
         return tuple(reduce)
 
     def __setstate__(self, state):
-        #print("in setstate %s" % str(state))
-        super(ArrayWithUnit, self).__setstate__(state["np_state"])
+        # pylint: disable=E1101
+        super().__setstate__(state["np_state"])
         self._unit = state["_unit"]
 
     def __repr__(self):
@@ -567,14 +568,12 @@ class ArrayWithUnit(np.ndarray):
     def __add__(self, other):
         if hasattr(other, "unit_type"):
             if other.unit_type != self.unit_type:
-                raise UnitError("Adding different types of units is"
-                                " not allowed")
+                raise UnitError("Adding different types of units is" " not allowed")
 
             if other.unit != self.unit:
                 other = other.to(self.unit)
 
-        return self.__class__(np.array(self) + np.array(other),
-                              unit_type=self.unit_type, unit=self.unit)
+        return self.__class__(np.array(self) + np.array(other), unit_type=self.unit_type, unit=self.unit)
 
     def __sub__(self, other):
         if hasattr(other, "unit_type"):
@@ -584,8 +583,7 @@ class ArrayWithUnit(np.ndarray):
             if other.unit != self.unit:
                 other = other.to(self.unit)
 
-        return self.__class__(np.array(self) - np.array(other),
-                              unit_type=self.unit_type, unit=self.unit)
+        return self.__class__(np.array(self) - np.array(other), unit_type=self.unit_type, unit=self.unit)
 
     def __mul__(self, other):
         # FIXME
@@ -600,45 +598,47 @@ class ArrayWithUnit(np.ndarray):
         # bit misleading.
         # Same protocol for __div__
         if not hasattr(other, "unit_type"):
-            return self.__class__(np.array(self).__mul__(np.array(other)),
-                                  unit_type=self._unit_type, unit=self._unit)
-        else:
-            # Cannot use super since it returns an instance of self.__class__
-            # while here we want a bare numpy array.
             return self.__class__(
                 np.array(self).__mul__(np.array(other)),
-                unit=self.unit * other.unit)
+                unit_type=self._unit_type,
+                unit=self._unit,
+            )
+        # Cannot use super since it returns an instance of self.__class__
+        # while here we want a bare numpy array.
+        return self.__class__(np.array(self).__mul__(np.array(other)), unit=self.unit * other.unit)
 
     def __rmul__(self, other):
+        # pylint: disable=E1101
         if not hasattr(other, "unit_type"):
-            return self.__class__(np.array(self).__rmul__(np.array(other)),
-                                  unit_type=self._unit_type, unit=self._unit)
-        else:
             return self.__class__(
                 np.array(self).__rmul__(np.array(other)),
-                unit=self.unit * other.unit)
+                unit_type=self._unit_type,
+                unit=self._unit,
+            )
+        return self.__class__(np.array(self).__rmul__(np.array(other)), unit=self.unit * other.unit)
 
     def __div__(self, other):
+        # pylint: disable=E1101
         if not hasattr(other, "unit_type"):
-            return self.__class__(np.array(self).__div__(np.array(other)),
-                                  unit_type=self._unit_type, unit=self._unit)
-        else:
             return self.__class__(
                 np.array(self).__div__(np.array(other)),
-                unit=self.unit/other.unit)
+                unit_type=self._unit_type,
+                unit=self._unit,
+            )
+        return self.__class__(np.array(self).__div__(np.array(other)), unit=self.unit / other.unit)
 
     def __truediv__(self, other):
+        # pylint: disable=E1101
         if not hasattr(other, "unit_type"):
-            return self.__class__(np.array(self).__truediv__(np.array(other)),
-                                  unit_type=self._unit_type, unit=self._unit)
-        else:
             return self.__class__(
                 np.array(self).__truediv__(np.array(other)),
-                unit=self.unit / other.unit)
+                unit_type=self._unit_type,
+                unit=self._unit,
+            )
+        return self.__class__(np.array(self).__truediv__(np.array(other)), unit=self.unit / other.unit)
 
     def __neg__(self):
-        return self.__class__(np.array(self).__neg__(),
-                              unit_type=self.unit_type, unit=self.unit)
+        return self.__class__(np.array(self).__neg__(), unit_type=self.unit_type, unit=self.unit)
 
     def to(self, new_unit):
         """
@@ -658,9 +658,21 @@ class ArrayWithUnit(np.ndarray):
         """
         return self.__class__(
             np.array(self) * self.unit.get_conversion_factor(new_unit),
-            unit_type=self.unit_type, unit=new_unit)
+            unit_type=self.unit_type,
+            unit=new_unit,
+        )
 
-    #TODO abstract base class property?
+    @property
+    def as_base_units(self):
+        """
+        Returns this ArrayWithUnit in base SI units, including derived units.
+
+        Returns:
+            An ArrayWithUnit object in base SI units
+        """
+        return self.to(self.unit.as_base_units[0])
+
+    # TODO abstract base class property?
     @property
     def supported_units(self):
         """
@@ -668,7 +680,7 @@ class ArrayWithUnit(np.ndarray):
         """
         return ALL_UNITS[self.unit_type]
 
-    #TODO abstract base class method?
+    # TODO abstract base class method?
     def conversions(self):
         """
         Returns a string showing the available conversions.
@@ -679,8 +691,9 @@ class ArrayWithUnit(np.ndarray):
 
 def _my_partial(func, *args, **kwargs):
     """
-    Partial returns a partial object and therefore we cannot inherit class methods defined in FloatWithUnit. 
-    This function calls partial and patches the new class before returning.
+    Partial returns a partial object and therefore we cannot inherit class
+    methods defined in FloatWithUnit. This function calls partial and patches
+    the new class before returning.
     """
     newobj = partial(func, *args, **kwargs)
     # monkey patch
@@ -752,7 +765,6 @@ Args:
 """
 ChargeArray = partial(ArrayWithUnit, unit_type="charge")
 
-
 Memory = _my_partial(FloatWithUnit, unit_type="memory")
 """
 A float with a memory unit.
@@ -777,18 +789,19 @@ def obj_with_unit(obj, unit):
 
     if isinstance(obj, numbers.Number):
         return FloatWithUnit(obj, unit=unit, unit_type=unit_type)
-    elif isinstance(obj, collections.Mapping):
-        return {k: obj_with_unit(v, unit) for k,v in obj.items()}
-    else:
-        return ArrayWithUnit(obj, unit=unit, unit_type=unit_type)
+    if isinstance(obj, collections.Mapping):
+        return {k: obj_with_unit(v, unit) for k, v in obj.items()}
+    return ArrayWithUnit(obj, unit=unit, unit_type=unit_type)
 
 
 def unitized(unit):
     """
-    Useful decorator to assign units to the output of a function. For
-    sequences, all values in the sequences are assigned the same unit. It
-    works with Python sequences only. The creation of numpy arrays loses all
-    unit information. For mapping types, the values are assigned units.
+    Useful decorator to assign units to the output of a function. You can also
+    use it to standardize the output units of a function that already returns
+    a FloatWithUnit or ArrayWithUnit. For sequences, all values in the sequences
+    are assigned the same unit. It works with Python sequences only. The creation
+    of numpy arrays loses all unit information. For mapping types, the values
+    are assigned units.
 
     Args:
         unit: Specific unit (eV, Ha, m, ang, etc.).
@@ -800,18 +813,21 @@ def unitized(unit):
             return 123.45
 
     """
+
     def wrap(f):
         def wrapped_f(*args, **kwargs):
             val = f(*args, **kwargs)
-            #print(val)
             unit_type = _UNAME2UTYPE[unit]
-            if isinstance(val, collections.Sequence):
+
+            if isinstance(val, (FloatWithUnit, ArrayWithUnit)):
+                return val.to(unit)
+
+            if isinstance(val, collections.abc.Sequence):
                 # TODO: why don't we return a ArrayWithUnit?
                 # This complicated way is to ensure the sequence type is
                 # preserved (list or tuple).
-                return val.__class__([FloatWithUnit(i, unit_type=unit_type,
-                                                    unit=unit) for i in val])
-            elif isinstance(val, collections.Mapping):
+                return val.__class__([FloatWithUnit(i, unit_type=unit_type, unit=unit) for i in val])
+            if isinstance(val, collections.abc.Mapping):
                 for k, v in val.items():
                     val[k] = FloatWithUnit(v, unit_type=unit_type, unit=unit)
             elif isinstance(val, numbers.Number):
@@ -821,10 +837,13 @@ def unitized(unit):
             else:
                 raise TypeError("Don't know how to assign units to %s" % str(val))
             return val
+
         return wrapped_f
+
     return wrap
 
 
 if __name__ == "__main__":
     import doctest
+
     doctest.testmod()
