@@ -1,4 +1,8 @@
-from __future__ import unicode_literals
+# coding: utf-8
+# Copyright (c) Pymatgen Development Team.
+# Distributed under the terms of the MIT License.
+
+
 """
 The util package implements various utilities that are commonly used by various
 packages.
